# -*- coding: utf-8 -*-

from SciDataTool import DataLinspace, DataTime, DataFreq, Data1D

from mosqito.functions.loudness_zwicker.loudness_zwicker_stationary import (
    loudness_zwicker_stationary,
)
from mosqito.functions.loudness_zwicker.loudness_zwicker_time import (
    loudness_zwicker_time,
)


def compute_loudness(self, field_type="free"):
    """Method to compute the loudness according to Zwicker's method

    Parameter
    ----------
    field-type: string
        'free' by default or 'diffuse'

    """

    # Compute third octave band spetrum if necessary
    if self.third_spec == None:
        self.comp_3oct_spec()

    # define bark_axis
    barks = DataLinspace(
        name="cr_band",
        unit="Bark",
        initial=0.1,
        final=24,
        number=int(24 / 0.1),
        include_endpoint=True,
    )

    if self.is_stationary:
        third_spec_data = self.third_spec.get_along("freqs", unit="dB")
        N, N_specific = loudness_zwicker_stationary(
            third_spec_data["x"],
            third_spec_data["freqs"],
            field_type,
        )
        self.loudness_zwicker = N
        axes = [barks]
    else:
        N, N_specific = loudness_zwicker_time(
            self.third_spec.get_along("freqs", "time", unit="dB")["x"], field_type
        )
        # Get time axis
        # Decimation from temporal resolution 0.5 ms to 2ms
        time = Data1D(
            name="time", unit="s", values=self.third_spec.get_axes()[1].values[::4]
        )
        self.loudness_zwicker = DataTime(
            name="Loudness",
            symbol="N_{zw}",
            unit="sone",
            axes=[time],
            values=N,
        )
        axes = [barks, time]

    self.loudness_zwicker_specific = DataFreq(
        name="Specific Loudness",
        symbol="N'_{zw}",
        unit="sone/Bark",
        axes=axes,
        values=N_specific,
    )



# -*- coding: utf-8 -*-
"""
Created on Mon Nov  9 10:31:09 2020

@author: wantysal
"""
# Standard library import
import numpy as np


def adaptlevel(signal, dBin, dBout):
    """ Scales a given signal to the demanded digital level in dB SPL [ref 2e-5] """

    # Calculate the factor to use for adapting the signal
    factor = np.power(10, (dBout - dBin) / 20)

    # Adapt the signal
    signal_scaled = signal * factor

    return signal_scaled



# -*- coding: utf-8 -*-
"""
Created on Mon Oct 19 13:41:37 2020

@author: wantysal
"""

# Standard imports
import pytest

# Local application imports
from mosqito.functions.roughness_danielweber.comp_roughness import comp_roughness
from tests.roughness.signals_test_generation import signal_test


@pytest.mark.roughness_dw  # to skip or run only Daniel and Weber roughness tests
def test_roughness():
    """Test function for the roughness calculation of a audio signal

    Test function for the script "comp_roughness" method with signal array
    as input. The input signals are chosen according to the article "Psychoacoustical
    roughness: implementation of an optimized model" by Daniel and Weber in 1997.
    The figure 3 is used to compare amplitude-modulated signals created according to
    their carrier frequency and modulation frequency to the article results.
    One .png compliance plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """

    # Stimulus generation
    stimulus = signal_test(fc=1000, fmod=70, mdepth=1, fs=44100, d=0.2, dB=60)

    # Roughness calculation
    R = comp_roughness(stimulus, fs=44100, overlap=0)

    # Check compliance
    tst = check_compliance(R)

    assert tst


def check_compliance(R):
    """Check the compliance of roughness calc. to Daniel and Weber article
    "Psychoacoustical roughness: implementation of an optimized model", 1997.

    Check the compliance of the input data R to figure 3 of the article
    using the reference data described in the dictionary article_ref.

    Parameter
    ---------
    R: numpy.array
        Calculated roughnesses [asper]

    Output
    ------
    tst : bool
        Compliance to the reference data
    """

    # Reference value of 1 asper given by Zwicker and Fastl
    ref = 1

    # Test for comformance (17% tolerance)
    tst = (R["values"] >= ref * 0.83).all() and (R["values"] <= ref * 1.17).all()

    return tst


# test de la fonction
if __name__ == "__main__":
    test_roughness()



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 30 14:25:04 2020

@author: wantysal
"""

# Standard library import
import numpy as np
import math

# Local imports
from mosqito.functions.tonality_tnr_pr.pr_main_calc import pr_main_calc


def comp_pr(is_stationary, signal, fs, prominence=True):
    """Computation of prominence ratio according to ECMA-74, annex D.10
        The T-PR value is calculated according to ECMA-TR/108

    Parameters
    ----------
    is_stationary : boolean
        True if the signal is stationary
    signal :numpy.array
        time signal values
    fs : integer
        sampling frequency
    prominence : boolean
        if True, the algorithm only returns the prominent tones, if False it returns all tones detected
    plot : str
        'y' to plot the results, 'n' to only return the dict

    Output
    ------
    output = dict
    {    "name" : "tone-to-noise ratio",
         "time" : np.linspace(0, len(signal)/fs, num=nb_frame),
         "freqs" : <frequency of the tones>
         "values" : <PR calculated value for each tone>
         "prominence" : <True or False according to ECMA criteria>
         "global value" : <sum of the specific TNR values>
            }


    """

    # Prominence criteria
    freqs = np.arange(90, 11200, 100)
    limit = np.zeros((len(freqs)))
    for i in range(len(freqs)):
        if freqs[i] >= 89.1 and freqs[i] < 1000:
            limit[i] = 9 + 10 * np.log10(1000 / freqs[i])
        if freqs[i] >= 1000 and freqs[i] < 11200:
            limit[i] = 9

    if is_stationary == True:
        tones_freqs, pr, prom, t_pr = pr_main_calc(signal, fs)
        tones_freqs = tones_freqs.astype(int)

        if prominence == True:
            output = {
                "name": "tone-to-noise ratio",
                "freqs": tones_freqs[prom],
                "values": pr[prom],
                "prominence": True,
                "global value": t_pr,
            }

        else:
            output = {
                "name": "tone-to-noise ratio",
                "freqs": tones_freqs,
                "values": pr,
                "prominence": prom,
                "global value": t_pr,
            }

    elif is_stationary == False:
        # Signal cut in frames of 500 ms along the time axis
        n = 0.5 * fs
        nb_frame = math.floor(signal.size / n)
        time = np.linspace(0, len(signal) / fs, num=nb_frame)
        time = np.around(time, 1)

        # Initialization of the result arrays
        tones_freqs = np.zeros((nb_frame), dtype=list)
        pr = np.zeros((nb_frame), dtype=list)
        prom = np.zeros((nb_frame), dtype=list)
        t_pr = np.zeros((nb_frame))

        # Compute PR values along time
        for i_frame in range(nb_frame):
            segment = signal[int(i_frame * n) : int(i_frame * n + n)]
            (
                tones_freqs[i_frame],
                pr[i_frame],
                prom[i_frame],
                t_pr[i_frame],
            ) = pr_main_calc(segment, fs)

        # Store the results in a time vs frequency array
        freq_axis = np.logspace(np.log10(90), np.log10(11200), num=1000)
        results = np.zeros((len(freq_axis), nb_frame))
        promi = np.zeros((len(freq_axis), nb_frame), dtype=bool)

        if prominence == True:
            for t in range(nb_frame):
                for f in range(len(tones_freqs[t])):
                    if prom[t][f] == True:
                        ind = np.argmin(np.abs(freq_axis - tones_freqs[t][f]))
                        results[ind, t] = pr[t][f]
                        promi[ind, t] = True
        else:
            for t in range(nb_frame):
                for f in range(len(tones_freqs[t])):
                    ind = np.argmin(np.abs(freq_axis - tones_freqs[t][f]))
                    results[ind, t] = pr[t][f]
                    promi[ind, t] = prom[t][f]

        output = {
            "name": "prominence ratio",
            "time": time,
            "freqs": freq_axis,
            "values": results,
            "prominence": promi,
            "global value": t_pr,
        }

    return output



# -*- coding: utf-8 -*-
"""
@date Created on Thur Apr 30 2020
@author martin_g for Eomys
"""

# Standard library imports
import numpy as np


def calc_main_loudness(spec_third, field_type):
    """Calculate core loudness

    The code is based on BASIC program published in "Program for
    calculating loudness according to DIN 45631 (ISO 532B)", E.Zwicker
    and H.Fastl, J.A.S.J (E) 12, 1 (1991).
    It also corresponds to the following functions of the C program
    published with ISO 532-1:2017:
    - corr_third_octave_intensities
    - f_calc_lcbs
    - f_calc_core_loudness
    - f_corr_loudness

    Parameters
    ----------
    spec_third : numpy.ndarray
        A third octave band spectrum [dB ref. 2e-5 Pa]
    field_type : str
        Type of soundfield correspondin to spec_third ("free" or
        "diffuse")

    Outputs
    -------
    nm :  numpy.ndarray
        Core loudness
    """
    #
    # Date tables definition (variable names and description according to
    # Zwicker:1991)
    # Ranges of 1/3 octave band levels for correction at low frequencies
    # according to equal loudness contours
    rap = np.array([45, 55, 65, 71, 80, 90, 100, 120])
    # Reduction of 1/3 octave band levels at low frequencies according to
    # equal loudness contours within the eight ranges defined by RAP
    dll = np.array(
        [
            (-32, -24, -16, -10, -5, 0, -7, -3, 0, -2, 0),
            (-29, -22, -15, -10, -4, 0, -7, -2, 0, -2, 0),
            (-27, -19, -14, -9, -4, 0, -6, -2, 0, -2, 0),
            (-25, -17, -12, -9, -3, 0, -5, -2, 0, -2, 0),
            (-23, -16, -11, -7, -3, 0, -4, -1, 0, -1, 0),
            (-20, -14, -10, -6, -3, 0, -4, -1, 0, -1, 0),
            (-18, -12, -9, -6, -2, 0, -3, -1, 0, -1, 0),
            (-15, -10, -8, -4, -2, 0, -3, -1, 0, -1, 0),
        ]
    )
    # Critical band level at absolute threshold without taking into
    # account the transmission characteristics of the ear
    ltq = np.array([30, 18, 12, 8, 7, 6, 5, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3])
    # Correction of levels according to the transmission characteristics
    # of the ear
    a0 = np.array(
        [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.5, -1.6, -3.2, -5.4, -5.6, -4, -1.5, 2, 5, 12]
    )
    # Level difference between free and diffuse sound fields
    ddf = np.array(
        [
            0,
            0,
            0.5,
            0.9,
            1.2,
            1.6,
            2.3,
            2.8,
            3,
            2,
            0,
            -1.4,
            -2,
            -1.9,
            -1,
            0.5,
            3,
            4,
            4.3,
            4,
        ]
    )
    # Adaptation of 1/3 oct. band levels to the corresponding critical
    # band level
    dcb = np.array(
        [
            -0.25,
            -0.6,
            -0.8,
            -0.8,
            -0.5,
            0,
            0.5,
            1.1,
            1.5,
            1.7,
            1.8,
            1.8,
            1.7,
            1.6,
            1.4,
            1.2,
            0.8,
            0.5,
            0,
            -0.5,
        ]
    )
    #
    # Correction of 1/3 oct. band levels according to equal loudness
    # contours 'xp' and calculation of the intensities for 1/3 oct.
    # bands up to 315 Hz
    ti = np.zeros((dll.shape[1], 1))
    for i in np.arange(dll.shape[1]):
        j = 0
        while spec_third[i] > rap[j] - dll[j, i] and j < dll.shape[0]:
            j += 1
        xp = spec_third[i] + dll[j, i]
        ti[i] = np.power(10, (xp / 10))

    # Determination of levels LCB(1), LCB(2) and LCB(3) within the
    # first three critical bands
    gi = np.zeros(3)
    gi[0] = ti[0:6].sum()
    gi[1] = ti[6:9].sum()
    gi[2] = ti[9:11].sum()
    lcb = np.zeros(3)
    lcb[gi > 0] = 10 * np.log10(gi[gi > 0])

    # Calculation of main loudness
    s = 0.25
    nm = np.zeros(20)
    le = spec_third[8:]
    le = le.reshape((20))
    le[0:3] = lcb
    le = le - a0
    if field_type == "diffuse":
        le += ddf
    i = le > ltq
    le[i] -= dcb[i]
    mp1 = 0.0635 * np.power(10, 0.025 * ltq[i])
    mp2 = np.power(1 - s + s * np.power(10, 0.1 * (le[i] - ltq[i])), 0.25) - 1
    nm[i] = mp1 * mp2
    nm[nm < 0] = 0
    nm = np.append(nm, 0)
    #
    # Correction of specific loudness in the lowest critical band
    # taking into account the dependance of absolute threshold
    # within this critical band
    korry = 0.4 + 0.32 * nm[0] ** 0.2
    if korry <= 1:
        nm[0] *= korry
    return nm


def calc_slopes(nm):
    """"""
    # Upper limits of approximated critical bands in terms of critical
    # band rate
    zup = np.array(
        [
            0.9,
            1.8,
            2.8,
            3.5,
            4.4,
            5.4,
            6.6,
            7.9,
            9.2,
            10.6,
            12.3,
            13.8,
            15.2,
            16.7,
            18.1,
            19.3,
            20.6,
            21.8,
            22.7,
            23.6,
            24,
        ]
    )
    # Range of specific loudness for the determination of the steepness
    # of the upper slopes in the specific loudness - critical band rate
    # pattern
    rns = np.array(
        [
            21.5,
            18,
            15.1,
            11.5,
            9,
            6.1,
            4.4,
            3.1,
            2.13,
            1.36,
            0.82,
            0.42,
            0.30,
            0.22,
            0.15,
            0.10,
            0.035,
            0,
        ]
    )
    # Steepness of the upper slopes in the specific loudness = Critical
    # band rate pattern for the ranges 'rns' as a function of the number
    # of the critical band
    usl = np.array(
        [
            (13, 8.2, 6.3, 5.5, 5.5, 5.5, 5.5, 5.5),
            (9, 7.5, 6, 5.1, 4.5, 4.5, 4.5, 4.5),
            (7.8, 6.7, 5.6, 4.9, 4.4, 3.9, 3.9, 3.9),
            (6.2, 5.4, 4.6, 4.0, 3.5, 3.2, 3.2, 3.2),
            (4.5, 3.8, 3.6, 3.2, 2.9, 2.7, 2.7, 2.7),
            (3.7, 3.0, 2.8, 2.35, 2.2, 2.2, 2.2, 2.2),
            (2.9, 2.3, 2.1, 1.9, 1.8, 1.7, 1.7, 1.7),
            (2.4, 1.7, 1.5, 1.35, 1.3, 1.3, 1.3, 1.3),
            (1.95, 1.45, 1.3, 1.15, 1.1, 1.1, 1.1, 1.1),
            (1.5, 1.2, 0.94, 0.86, 0.82, 0.82, 0.82, 0.82),
            (0.72, 0.67, 0.64, 0.63, 0.62, 0.62, 0.62, 0.62),
            (0.59, 0.53, 0.51, 0.50, 0.42, 0.42, 0.42, 0.42),
            (0.40, 0.33, 0.26, 0.24, 0.24, 0.22, 0.22, 0.22),
            (0.27, 0.21, 0.20, 0.18, 0.17, 0.17, 0.17, 0.17),
            (0.16, 0.15, 0.14, 0.12, 0.11, 0.11, 0.11, 0.11),
            (0.12, 0.11, 0.10, 0.08, 0.08, 0.08, 0.08, 0.08),
            (0.09, 0.08, 0.07, 0.06, 0.06, 0.06, 0.06, 0.05),
            (0.06, 0.05, 0.03, 0.02, 0.02, 0.02, 0.02, 0.02),
        ]
    )

    #
    # Start values
    j = 0
    N = 0
    z1 = 0
    n1 = 0
    iz = 0
    z = 0.1
    N_specific = np.zeros(int(24 / 0.1))
    #
    # Step to first and subsequent critical bands
    for i in np.arange(21):
        zup[i] += 0.0001
        ig = i - 1
        if ig > 7:
            ig = 7
        while z1 < zup[i]:
            if n1 <= nm[i]:
                if n1 < nm[i]:
                    #
                    # Determination of the number j corresponding to the range
                    # of specific loudness
                    j = 0
                    while rns[j] > nm[i] and j < 17:
                        j += 1
                #
                # Contribution of unmasked main loudness to total loudness
                # and calculation of values N_specific(iz) with a spacing of
                # z = iz * 0.1 bark
                z2 = zup[i]
                n2 = nm[i]
                N = N + n2 * (z2 - z1)
                while z < z2:
                    N_specific[iz] = n2
                    iz += 1
                    z += 0.1
            else:
                #
                # Decision wether the critical band in question is completely
                # or partially masked by accessory loudness
                n2 = rns[j]
                if n2 < nm[i]:
                    n2 = nm[i]
                dz = (n1 - n2) / usl[j, ig]
                z2 = z1 + dz
                if z2 > zup[i]:
                    z2 = zup[i]
                    dz = z2 - z1
                    n2 = n1 - dz * usl[j, ig]
                #
                # Contribution of accessory loudness to total loudness
                N = N + dz * (n1 + n2) / 2
                while z < z2:
                    N_specific[iz] = n1 - (z - z1) * usl[j, ig]
                    iz += 1
                    z += 0.1
            #
            # Step to next segment
            while n2 <= rns[j] and j < 17:
                j += 1
            if n2 <= rns[j] and j >= 17:
                j = 17
            z1 = z2
            n1 = n2
    #
    # Final correction
    if N < 0:
        N = 0
    if N <= 16:
        N = np.floor(N * 1000 + 0.5) / 1000
    else:
        N = np.floor(N * 100 + 0.5) / 100

    return N, N_specific



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 09:14:07 2020

@author: wantysal
"""

# Standard library import
from numpy import mean, sqrt, log10, linspace


def comp_level(signal, fs, nb_points=[], start=[], stop=[]):
    """Overall Sound Pressure Level calculation from the time signal
    The SPL can be computed according to a specified number of points or
    during a given time frame

    Parameter:
    ----------
    signal : numpy.array
        time signal value
    fs: integer
        sampling frequency

    Output:
    -------
    level : numpy.array
        SPL in dB

    """
    # Check the inputs
    if nb_points != []:
        if type(nb_points) != int:
            raise TypeError("ERROR : Number of points should be an integer")

        if nb_points < 1 or nb_points > len(signal):
            raise ValueError(
                "ERROR : Number of points should be between 1 and the length of the given signal"
            )

    if start != [] and stop != []:
        if type(start) != int and type(start) != float:
            raise TypeError("ERROR : Start (in sec) should be an integer or a float ")

        if type(stop) != int and type(stop) != float:
            raise TypeError("ERROR : Stop (in sec) should be an integer or a float ")

        if start < 0 or stop < 0 or start > len(signal) / fs or stop > len(signal) / fs:
            raise ValueError(
                "ERROR : Time frame should be between 0s and the duration of the signal"
            )

        if start == stop:
            raise ValueError("ERROR : Start and stop values must be different")

    # Initialization
    level = []

    # Case of a given time frame
    if start != [] and stop != []:
        frame = signal[int(start * fs) : int(stop * fs)]
    else:
        start = 0
        stop = len(signal) / fs
        frame = signal

    # Case of a given number of points
    if nb_points != []:

        time_axis = linspace(start, stop, num=nb_points)

        frame_size = int(len(frame) / nb_points)
        for i in range(nb_points):
            frame_i = frame[i * frame_size : i * frame_size + frame_size]
            peff = sqrt(mean(frame_i ** 2))
            level.append(10 * log10((peff ** 2 / (2e-05) ** 2)))

    else:
        peff = sqrt(mean(frame ** 2))
        level = 10 * log10((peff ** 2 / (2e-05) ** 2))

    output = {
        "name": "Sound Pressure Level",
        "values": level,
        "time": time_axis,
    }

    return output



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 30 15:25:12 2020

@author: wantysal
"""

# import SciDataTool objects
from SciDataTool import DataTime, DataLinspace

# import methods
from mosqito.functions.shared.load import load
from mosqito.methods.Audio.cut_signal import cut_signal
from mosqito.methods.Audio.comp_3oct_spec import comp_3oct_spec
from mosqito.methods.Audio.compute_welch import compute_welch
from mosqito.methods.Audio.compute_level import compute_level
from mosqito.methods.Audio.compute_loudness import compute_loudness
from mosqito.methods.Audio.compute_sharpness import compute_sharpness
from mosqito.methods.Audio.compute_roughness import compute_roughness
from mosqito.methods.Audio.compute_tnr_pr import compute_tnr_pr


class Audio:
    """Audio signal loading and analysis: from .wav or .uff files compute
    loudness, sharpness and roughness values thanks to the Mosqito package,
    Results plotting thanks to the SciDataTool package."""

    def __init__(self, file, is_stationary=False, calib=1, mat_signal="", mat_fs=""):
        """Constructor of the class. Loads the signal from a .wav .mat or .uff file

        Parameters
        ----------
        self : Audio object
            Object from the Audio class
        file : string
            string path to the signal file
        is_stationary : boolean
            TRUE if the signal is stationary, FALSE if it is time-varying
        calib : float
            calibration factor for the signal to be in [pa]
        mat_signal : string
            in case of a .mat file, name of the signal variable
        mat_fs : string
            in case of a .mat file, name of the sampling frequency variable

        """

        # Import audio signal
        values, fs = load(
            is_stationary,
            file,
            calib=calib,
            mat_signal=mat_signal,
            mat_fs=mat_fs,
        )

        # Create Data object for time axis
        time_axis = DataLinspace(
            name="time",
            unit="s",
            initial=0,
            final=(len(values) - 1) / fs,
            number=len(values),
            include_endpoint=True,
        )

        # Create audio signal Data object and populate the object
        self.fs = fs
        self.is_stationary = is_stationary
        self.signal = DataTime(
            name="Audio signal",
            symbol="x",
            unit="Pa",
            normalizations={"ref": 2e-5},
            axes=[time_axis],
            values=values,
        )

        # Init physical metrics attributes
        self.third_spec = None
        self.level = None
        self.welch = None

        # Init physiological metrics attributes
        self.loudness_zwicker = None
        self.loudness_zwicker_specific = None
        self.sharpness = dict()
        self.roughness = dict()
        self.tonality = dict()

    # Methods
    cut_signal = cut_signal
    comp_3oct_spec = comp_3oct_spec
    compute_welch = compute_welch
    compute_level = compute_level
    compute_loudness = compute_loudness
    compute_sharpness = compute_sharpness
    compute_roughness = compute_roughness
    compute_tnr_pr = compute_tnr_pr



# -*- coding: utf-8 -*-

from mosqito.functions.shared.load import load
from SciDataTool import DataTime, DataLinspace


def import_signal(self, is_stationary, file, calib=1, mat_signal="", mat_fs=""):
    """Method to load the signal from a .wav .mat or .uff file

    Parameters
    ----------
    self : Audio object
        Object from the Audio class
    is_stationary : boolean
        TRUE if the signal is stationary, FALSE if it is time-varying
    file : string
        string path to the signal file
    calib : float
        calibration factor for the signal to be in [pa]
    mat_signal : string
        in case of a .mat file, name of the signal variable
    mat_fs : string
        in case of a .mat file, name of the sampling frequency variable


    Outputs
    -------
    signal : numpy.array
        time signal values
    fs : integer
        sampling frequency

    """

    # Init Audio object
    self.__init__()

    # Import audio signal
    values, fs = load(
        is_stationary,
        file,
        calib=calib,
        mat_signal=mat_signal,
        mat_fs=mat_fs,
    )

    # Create Data object for time axis
    time_axis = DataLinspace(
        name="time",
        unit="s",
        initial=0,
        final=(len(values) - 1) / fs,
        number=len(values),
        include_endpoint=True,
    )

    # Create audio signal Data object
    self.fs = fs
    self.is_stationary = is_stationary
    self.signal = DataTime(
        name="Audio signal",
        symbol="x",
        unit="Pa",
        axes=[time_axis],
        values=values,
    )



# -*- coding: utf-8 -*-
"""
Created on Tue Feb 23 14:05:22 2021

@author: Salomé
"""
# Import SciDataTool objects

from SciDataTool import DataTime, Data1D

# Import MOSQITO function
from mosqito.functions.roughness_danielweber.comp_roughness import comp_roughness


def compute_roughness(self, method="dw", overlap=0.5):
    """Method to compute roughness according to the Daniel and Weber implementation

    Parameter
    ---------
    method : string
        method used to do the computation 'danielweber' is the only one for now
    overlap : float
        overlapping coefficient for the time windows of 200ms, default is 0.5
    """

    # check the input parameters
    if method != "dw":
        raise ValueError("ERROR: method must be 'dw'")

    if overlap < 0 or overlap > 1:
        raise ValueError("ERROR: overlap value must be between 0 and 1")

    R = comp_roughness(self.signal.values, self.fs, overlap)

    time = Data1D(name="time", unit="s", values=R["time"])

    self.roughness["Daniel Weber"] = DataTime(
        symbol="R_{dw}",
        axes=[time],
        values=R["values"],
        name="Roughness",
        unit="Asper",
    )



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 15 18:06:47 2021

@author: Mesure_07
"""
import pytest
from mosqito.classes.Audio import Audio
from mosqito import COLORS

out_path = "./tests/output/"
is_show_fig = False


@pytest.fixture(scope="module")
def fixture_import_signal():
    audio = Audio(
        "./tests/input/Test signal 3 (1 kHz 60 dB).wav",
        is_stationary=True,
        calib=2 * 2 ** 0.5,
    )
    return audio


@pytest.fixture(scope="module")
def fixture_import_signal_time():
    audio = Audio(
        "./tests/input/Test signal 6 (tone 250 Hz 30 dB - 80 dB).wav",
        calib=2 * 2 ** 0.5,
    )
    return audio


@pytest.mark.audio
def test_import_signal():
    audio = Audio(
        "./tests/input/Test signal 3 (1 kHz 60 dB).wav",
        is_stationary=True,
        calib=2 * 2 ** 0.5,
    )
    audio.signal.plot_2D_Data(
        "time",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_import_signal.png",
        color_list=COLORS,
    )


@pytest.mark.audio
def test_comp_3oct_spec():
    audio1 = Audio(
        "./tests/input/Test signal 3 (1 kHz 60 dB).wav",
        is_stationary=True,
        calib=2 * 2 ** 0.5,
    )
    audio1.comp_3oct_spec(fc_min=20, fc_max=20000)
    audio1.third_spec.plot_2D_Data(
        "freqs",
        type_plot="octave",
        is_logscale_x=True,
        # x_min=20,
        # y_min=0,
        # y_max=40,
        unit="dB",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_comp_3oct_spec.png",
    )


@pytest.mark.audio
def test_level(fixture_import_signal_time):
    audio = fixture_import_signal_time
    audio.compute_level(nb_points=20, start=0.5, stop=4)
    audio.level.plot_2D_Data(
        "time",
        type_plot="curve",
        unit="dB",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_level.png",
    )


@pytest.mark.audio
def test_compute_loudness(fixture_import_signal):
    audio = fixture_import_signal
    audio.compute_loudness()
    audio.loudness_zwicker_specific.plot_2D_Data(
        "cr_band",
        type_plot="curve",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_loudness.png",
        color_list=COLORS,
    )


@pytest.mark.audio
def test_compute_loudness_time(fixture_import_signal_time):
    audio = fixture_import_signal_time
    audio.compute_loudness()
    audio.loudness_zwicker.plot_2D_Data(
        "time",
        type_plot="curve",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_loudness_time_1.png",
        color_list=COLORS,
    )
    audio.loudness_zwicker_specific.plot_2D_Data(
        "time",
        "cr_band=2.5",
        type_plot="curve",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_loudness_time_2.png",
        color_list=COLORS,
    )
    audio.loudness_zwicker_specific.plot_3D_Data(
        "cr_band",
        "time",
        is_2D_view=True,
        is_switch_axes=True,
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_loudness_time_3.png",
    )


@pytest.mark.audio
def test_compute_sharpness(fixture_import_signal):
    audio = fixture_import_signal
    audio.compute_sharpness(method="all")


@pytest.mark.audio
def test_compute_sharpness_time(fixture_import_signal_time):
    audio = fixture_import_signal_time
    audio.compute_sharpness(method="all")
    audio.sharpness["din"].plot_2D_Data(
        "time",
        type_plot="curve",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_sharpness_time.png",
        color_list=COLORS,
    )


@pytest.mark.audio
def test_compute_roughness(fixture_import_signal):
    audio = fixture_import_signal
    audio.compute_roughness(overlap=0)


@pytest.mark.audio
def test_compute_roughness_time(fixture_import_signal_time):
    audio = fixture_import_signal_time
    audio.compute_roughness(overlap=0)
    audio.roughness["Daniel Weber"].plot_2D_Data(
        "time",
        type_plot="curve",
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_roughness_time.png",
        color_list=COLORS,
    )


@pytest.mark.audio
def test_compute_tnr_pr(fixture_import_signal):
    audio = fixture_import_signal
    audio.compute_tnr_pr(method="all")


@pytest.mark.audio
def test_compute_tnr_pr_time(fixture_import_signal_time):
    audio = fixture_import_signal_time
    audio.compute_tnr_pr(method="all")
    audio.tonality["tnr"].plot_3D_Data(
        "time",
        "freqs",
        is_2D_view=True,
        is_show_fig=is_show_fig,
        save_path=out_path + "test_Audio_compute_tnr_pr_time.png",
    )


from scipy import signal
import numpy as np
import matplotlib.pyplot as plt

"""
Script shared by https://www.encida.dk
ISO61260
"""


def getFrequencies(fstart, fend, b, G=10, fr=1000):
    """
    Calculate octave filter specifications
    :param fstart: start frequency
    :param fend: end frequency
    :param b: number of bands pr octave
    :param G: base
    :param fr: reference frequency
    :return: filter specification dict
    """

    # frequency matrix
    freqs = np.empty((0, 3))

    if G == 10:
        G = 10 ** (3 / 10)  # Base ten
    elif G == 2:
        G = 2
    else:
        print("The base system is not permitted. G must be 10 or 2")
        # raise

    x = -1000
    f2 = 0
    while f2 <= fend:
        # Excact midband frequencies
        if b % 2 == 0:  # even
            fm = (G ** ((2 * x - 59) / (2 * b))) * (fr)
        else:  # odd
            fm = (G ** ((x - 30) / b)) * (fr)
        # Bandedge frequencies
        f1 = (G ** (-1 / (2 * b))) * (fm)
        f2 = (G ** (1 / (2 * b))) * (fm)
        if f2 >= fstart:
            freqs = np.append(freqs, np.array([[f1, fm, f2]]), axis=0)
        x += 1
    return {"f": freqs, "b": b, "G": G}


def designFilters(freqDict, fs=48000, plot=False):
    """
    Design octave band filters
    :param freqDict: filter specification dict
    :param fs: sample rate
    :param plot: bool if filters should be plotted
    :return: filter coefficients
    """

    freqs = freqDict["f"]
    b = freqDict["b"]
    G = freqDict["G"]

    order = 4
    filters = np.empty((0, 6))
    for index in range(np.size(freqs, axis=0)):
        lowCutoff = 2 * freqs[index, 0] / fs
        highCutoff = 2 * freqs[index, 2] / fs
        sos = signal.butter(
            order, [lowCutoff, highCutoff], btype="bandpass", output="sos"
        )
        filters = np.append(filters, sos, axis=0)

    if plot is True:
        # Requirements
        maskLevelmax = -np.array([0.15, 0.2, 0.4, 1.1, 4.5, 4.5, 200, 200, 200, 200])
        maskLevelmin = -np.array(
            [-0.15, -0.15, -0.15, -0.15, -0.15, 2.3, 18, 42.5, 62, 75]
        )
        breakpoints = np.array([0, 1 / 8, 1 / 4, 3 / 8, 1 / 2, 1 / 2, 1, 2, 3, 4])

        freqH = 1 + ((G ** (1 / (2 * b)) - 1) / (G ** (1 / 2) - 1)) * (
            G ** (breakpoints) - 1
        )
        freqL = 1 / freqH

        plt.figure(
            "Filter responses with {} bands from {} Hz to {} Hz".format(
                np.size(freqs, 0), np.int(freqs[0, 0]), np.int(freqs[-1, 2])
            )
        )
        for index in range(np.size(freqs, axis=0)):
            # Plot
            w, h = signal.sosfreqz(
                filters[(order * index) : (order * index + order), :], worN=fs
            )
            plt.semilogx(freqH * freqs[index, 1], maskLevelmax, "k--")
            plt.semilogx(freqH * freqs[index, 1], maskLevelmin, "k--")
            plt.semilogx(freqL * freqs[index, 1], maskLevelmax, "k--")
            plt.semilogx(freqL * freqs[index, 1], maskLevelmin, "k--")
            plt.semilogx((fs * 0.5 / np.pi) * w, 20 * np.log10(abs(h)))
            plt.ylim([-100, 1])
        plt.show()
    return filters


def filterData(filters, data):
    """
    Filter data using octave filters
    :param filters: octave filter coefficients
    :param data: data that should be filtered
    :return: filtered data
    """

    order = 4
    # Construct signal
    if data.ndim == 1:
        filteredSignal = np.zeros(
            [np.size(data, axis=0), int(np.size(filters, 0) / order)]
        )
        for index in range(int(np.size(filters, axis=0) / order)):
            filteredSignal[:, index] = signal.sosfilt(
                filters[(order * index) : (order * index + order), :], data
            )
    elif data.ndim == 2:
        filteredSignal = np.zeros(
            [
                np.size(data, axis=0),
                int(np.size(filters, 0) / order),
                np.size(data, axis=1),
            ]
        )
        for dataIndex in range(np.size(data, axis=1)):
            for bandIndex in range(int(np.size(filters, axis=0) / order)):
                filteredSignal[:, bandIndex, dataIndex] = signal.sosfilt(
                    filters[(order * bandIndex) : (order * bandIndex + order), :],
                    data[:, dataIndex],
                )
    return filteredSignal


def analyseData(filters, data, freqDict=None, plot=False):
    """
    Octave band analyse
    :param filters: octave filter coefficients
    :param data: data to be analysed
    :param freqDict: filter specification dict
    :param plot: bool if analysis should be plotted
    :return:
    """

    filteredSignal = filterData(filters, data)
    meanSignal = np.sqrt(np.mean(filteredSignal ** 2, axis=0))
    if plot == 1:
        minMean = np.min(np.min(20 * np.log10(meanSignal), axis=0) - 5, axis=0)
        freqs = freqDict["f"]
        fig, ax = plt.subplots()
        if meanSignal.ndim == 1:
            ax.bar(
                freqs[:, 0],
                20 * np.log10(meanSignal) - minMean,
                align="edge",
                bottom=minMean,
                width=(freqs[:, 2] - freqs[:, 0]),
            )
        elif meanSignal.ndim == 2:
            for i in range(np.size(meanSignal, axis=1)):
                ax.bar(
                    freqs[:, 0],
                    20 * np.log10(meanSignal[:, i]) - minMean,
                    align="edge",
                    bottom=minMean,
                    width=(freqs[:, 2] - freqs[:, 0]),
                )
        ax.set_xscale("log")
        plt.show()
    return meanSignal


# if __name__ == "__main__":
#     fs = 48000

#     freqs = getFrequencies(60, 20000, 1)
#     filters = designFilters(freqs, fs, plot=False)
#     # data
#     t = np.arange(0, 1, 1 / fs)
#     s = np.sin(2 * np.pi * 1000 * t)
#     # filter
#     filteredSignal = filterData(filters, s)

#     # analyse
#     rmsData = analyseData(filters, s, freqs, plot=False)

#     #plot
#     plt.figure('Results')
#     plt.subplot(1, 2, 1)
#     plt.title('Filtered signal')
#     plt.plot(filteredSignal)
#     plt.subplot(1, 2, 2)
#     plt.title('RMS signal')
#     plt.step(freqs['f'][:,1],rmsData)
#     plt.xscale('log')
#     plt.show()



# -*- coding: utf-8 -*-
"""
@date Created on Mon Mar 23 2020
@author martin_g for Eomys
"""

# Third party imports
import numpy as np
import matplotlib.pyplot as plt
import pytest

# Local application imports
from mosqito.functions.loudness_zwicker.loudness_zwicker_stationary import (
    loudness_zwicker_stationary,
)
from mosqito.functions.shared.load import load2oct3


@pytest.mark.loudness_zwst  # to skip or run only loudness zwicker stationary tests
def test_loudness_zwicker_3oct():
    """Test function for the script loudness_zwicker_stationary

    Test function for the script loudness_zwicker_stationary with
    third octave band spectrum as input. The input spectrum is
    provided by ISO 532-1 annex B2, the compliance is assessed
    according to section 5.1 of the standard. One .png compliance
    plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Third octave levels as input for stationary loudness
    # (from ISO 532-1 annex B2)
    test_signal_1 = np.array(
        [
            -60,
            -60,
            78,
            79,
            89,
            72,
            80,
            89,
            75,
            87,
            85,
            79,
            86,
            80,
            71,
            70,
            72,
            71,
            72,
            74,
            69,
            65,
            67,
            77,
            68,
            58,
            45,
            30.0,
        ]
    )

    signal = {
        "data_file": "Test signal 1.txt",
        "N": 83.296,
        "N_specif_file": "tests/input/test_signal_1.csv",
    }
    N, N_specific = loudness_zwicker_stationary(test_signal_1)

    tst = check_compliance(N, N_specific, signal)

    assert tst


@pytest.mark.loudness_zwst  # to skip or run only loudness zwicker stationary tests
def test_loudness_zwicker_wav():
    """Test function for the script loudness_zwicker_stationary

    Test function for the script loudness_zwicker_stationary with
    .wav file as input. The input file is provided by ISO 532-1 annex
    B3, the compliance is assessed according to section 5.1 of the
    standard. One .png compliance plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Test signal as input for stationary loudness
    # (from ISO 532-1 annex B3)
    signal = {
        "data_file": "tests/input/Test signal 3 (1 kHz 60 dB).wav",
        "N": 4.019,
        "N_specif_file": "tests/input/test_signal_3.csv",
    }

    # Load signal and compute third octave band spectrum
    third_spec = load2oct3(True, signal["data_file"], calib=2 * 2 ** 0.5)

    # Compute Loudness
    N, N_specific = loudness_zwicker_stationary(third_spec["values"])

    # Check ISO 532-1 compliance
    assert check_compliance(N, N_specific, signal)


def check_compliance(N, N_specific, iso_ref):
    """Check the comppiance of loudness calc. to ISO 532-1

    Check the compliance of the input data N and N_specific
    to section 5.1 of ISO 532-1 by using the reference data
    described in dictionary iso_ref.

    Parameters
    ----------
    N : float
        Calculated loudness [sones]
    N_specific : numpy.ndarray
        Specific loudness [sones/bark]
    bark_axis : numpy.ndarray
        Corresponding bark axis
    iso_ref : dict
        {
            "data_file": <Path to reference input signal>,
            "N": <Reference loudness value>,
            "N_specif_file": <Path to reference calculated specific loudness>
        }
        Dictionary containing link to ref. data

    Outputs
    -------
    tst : bool
        Compliance to the reference data
    """
    # Load ISO reference outputs
    N_iso = iso_ref["N"]
    N_specif_iso = np.genfromtxt(iso_ref["N_specif_file"], skip_header=1)
    # Test for ISO 532-1 comformance (section 5.1)
    tst_N = (
        N >= N_iso * 0.95
        and N <= N_iso * 1.05
        and N >= N_iso - 0.1
        and N <= N_iso + 0.1
    )
    tst_specif = (
        N_specific >= np.amin([N_specif_iso * 0.95, N_specif_iso - 0.1], axis=0)
    ).all() and (
        N_specific <= np.amax([N_specif_iso * 1.05, N_specif_iso + 0.1], axis=0)
    ).all()
    tst = tst_N and tst_specif

    # Define and plot the tolerance curves
    bark_axis = np.linspace(0.1, 24, int(24 / 0.1))
    tol_curve_min = np.amin([N_specif_iso * 0.95, N_specif_iso - 0.1], axis=0)
    tol_curve_min[tol_curve_min < 0] = 0
    tol_curve_max = np.amax([N_specif_iso * 1.05, N_specif_iso + 0.1], axis=0)
    plt.plot(
        bark_axis,
        tol_curve_min,
        color="red",
        linestyle="solid",
        label="5% tolerance",
        linewidth=1,
    )
    plt.plot(
        bark_axis, tol_curve_max, color="red", linestyle="solid", label="", linewidth=1
    )
    plt.legend()

    # Compliance plot

    plt.plot(bark_axis, N_specific, label="MOSQITO")
    if tst_specif:
        plt.text(
            0.5,
            0.5,
            "Test passed (5% tolerance not exceeded)",
            horizontalalignment="center",
            verticalalignment="center",
            transform=plt.gca().transAxes,
            bbox=dict(facecolor="green", alpha=0.3),
        )
    else:
        tst = 0
        plt.text(
            0.5,
            0.5,
            "Test not passed",
            horizontalalignment="center",
            verticalalignment="center",
            transform=plt.gca().transAxes,
            bbox=dict(facecolor="red", alpha=0.3),
        )

    if tst_N:
        clr = "green"
    else:
        clr = "red"
    plt.title("N = " + str(N) + " sone (ISO ref. " + str(N_iso) + " sone)", color=clr)
    file_name = "_".join(iso_ref["data_file"].split(" "))
    plt.savefig(
        "tests/output/test_loudness_zwicker_wav_"
        + file_name.split("/")[-1][:-4]
        + ".png",
        format="png",
    )
    plt.clf()
    return tst


# test de la fonction
if __name__ == "__main__":
    test_loudness_zwicker_3oct()



# -*- coding: utf-8 -*-

from mosqito.functions.shared.cut import cut_signal as cut_signal_fct


def cut_signal(self, start, stop):
    """Method to keep only the signal values between 'start' and 'stop'

    TODO : replace by SciDataTool slicing

    Parameters
    ----------
    start : float
        beginning of the new signal in [s]
    stop : float
        end of the new signal in [s]

    """

    self.signal.values = cut_signal_fct(self.signal.values, self.fs, start, stop)



# -*- coding: utf-8 -*-
"""
@date Created on Tue Mar 03 2020
@author martin_g for Eomys
"""
# Standard library imports
import math
from scipy import signal


def oct3dsgn(fc, fs, n=3):
    """Design of a one-third-octave filter

    Designs a digital 1/3-octave filter with center frequency fc for
    sampling frequency fs. The filter is designed according to the
    Order-N specification of the ANSI S1.1-1986 standard. Default
    value for N is 3.

    References:
        ANSI S1.1-1986 (ASA 65-1986): Specifications for
        Octave-Band and Fractional-Octave-Band Analog and
        Digital Filters, 1993.

    Parameters
    ----------
    fc : float
        Filter center frequency [Hz]
    fs : float
        Sampling frequency [Hz]
    N : str
        Filter order according to ANSI S1.1-1986

    Outputs
    -------
    A : numpy.ndarray
        Denominator coefficient of the filter rational transfer function
    B : numpy.ndarray
        Numerator coefficient of the filter rational transfer function
    """

    """
    Initial implementation as oct3dsgn.m by Christophe Couvreur, 
    Faculte Polytechnique de Mons (Belgium) couvreur@thor.fpms.ac.be
    (version: Aug. 25, 1997, 2:00pm)

    TODO: Check compliancy with the following standards
        - IEC 61260 – 1 (2014), 1/1-octave Bands and 1/3-octave Bands, Class 1
        - IEC 61260 (1995 – 07) plus Amendment 1 (2001 – 09), 1/1-octave Bands and 1/3-octave Bands, Class 0
        - ANSI S1.11 – 1986, 1/1-octave Bands and 1/3-octave Bands, Order 3, Type 0 – C
        - ANSI S1.11 – 2004, 1/1-octave Bands and 1/3-octave Bands, Class 0
        - ANSI/ASA S1.11 – 2014 Part 1, 1/1-octave Bands and 1/3-octave Bands, Class 1
    """

    # Check for high fc/fs causing filter design issue [ref needed]
    if fc < fs / 200:
        raise ValueError(
            """ERROR: Design not possible. Filter center frequency shall
            verify: fc > fs / 200"""
        )

    # Check for Nyquist-Shannon criteria
    if fc > 0.88 * (fs / 2):
        raise ValueError(
            """ERROR: Design not possible. Filter center frequency shall
            verify: fc <= 0.88 * (fs / 2)"""
        )

    # Design Butterworth 2Nth-order one-third-octave filter
    # Note: BUTTER is based on a bilinear transformation, as suggested in
    # ANSI S1.1-1986.
    f1 = fc / (2 ** (1 / 6))
    f2 = fc * (2 ** (1 / 6))
    qr = fc / (f2 - f1)
    qd = (math.pi / 2 / n) / (math.sin(math.pi / 2 / n)) * qr
    alpha = (1 + math.sqrt(1 + 4 * qd ** 2)) / 2 / qd
    w1 = fc / (fs / 2) / alpha
    w2 = fc / (fs / 2) * alpha
    b, a = signal.butter(n, [w1, w2], "bandpass", analog=False)
    return b, a



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 30 15:12:18 2020

@author: wantysal
"""

import numpy as np
from numpy.fft import fft

# from scipy.signal import welch, periodogram

# Local functions imports
from mosqito.functions.shared.conversion import amp2db
from mosqito.functions.tonality_tnr_pr.critical_band import (
    critical_band,
    lower_critical_band,
    upper_critical_band,
)
from mosqito.functions.tonality_tnr_pr.screening_for_tones import screening_for_tones
from mosqito.functions.tonality_tnr_pr.find_highest_tone import find_highest_tone


def pr_main_calc(signal, fs):
    """
        Calculation of the tone-to noise ratio according to the method described
        in ECMA 74, annex D.
        The method to find the tonal candidates is the one described by Wade Bray
        in 'Methods for automating prominent tone evaluation and for considerin
        variations with time or other reference quantities'
        The total value calculated, T-PR, is calculated according to ECMA TR/108

    Parameters
    ----------
    signal : numpy.array
        time history values
    fs : integer
        sampling frequency

    Output
    ------
    tones_freqs : list of float
        frequency of the tones
    tnr : list of float
        TNR value calculated for each tone
    prominence : list of boolean
        prominence criteria as described in ECMA 74
    total_tnr : list of float
        sum of the specific TNR
    """

    #### Spectrum creation #######################################################

    n = len(signal)
    window = np.hanning(n)
    window = window / np.sum(window)

    # Creation of the spectrum by FFT
    spectrum = fft(signal * window) * 1.42

    # Conversion into dB level
    module = np.abs(spectrum)
    spectrum_db = amp2db(module, ref=0.00002)

    # Frequency axis of interest
    freq_axis = np.arange(0, int(n / 2), 1) * (fs / n)
    freq_index = np.where((freq_axis > 89.1) & (freq_axis < 11200))[0]
    freqs = freq_axis[freq_index]
    spec_db = spectrum_db[freq_index]

    #### Screening to find the potential tonal components ########################

    peak_index = screening_for_tones(freqs, spec_db, "smoothed", 90, 11200)
    nb_tones = len(peak_index)

    #### Evaluation of each candidate ############################################

    # Initialization of the results lists
    tones_freqs = np.array(())
    pr = np.array(())
    prominence = []
    # Each candidate is studied and then deleted from the list until all have been treated
    while nb_tones > 0:
        ind = peak_index[0]

        # Find the highest tone in the critical band
        if len(peak_index) > 1:
            ind, _, peak_index, nb_tones = find_highest_tone(
                freqs, spec_db, peak_index, nb_tones, ind
            )

        ft = freqs[ind]

        # Level of the middle critical band
        f1, f2 = critical_band(ft)
        low_limit_idx = np.argmin(np.abs(freqs - f1))
        high_limit_idx = np.argmin(np.abs(freqs - f2))

        spec_sum = sum(10 ** (spec_db[low_limit_idx:high_limit_idx] / 10))
        if spec_sum != 0:
            Lm = 10 * np.log10(spec_sum)
        else:
            Lm = 0

        # Level of the lower critical band
        f1, f2 = lower_critical_band(ft)
        low_limit = np.argmin(np.abs(freqs - f1))
        high_limit = np.argmin(np.abs(freqs - f2))

        spec_sum = sum(10 ** (spec_db[low_limit:high_limit] / 10))
        if spec_sum != 0:
            Ll = 10 * np.log10(spec_sum)
        else:
            Ll = 0

        delta_f = f2 - f1

        # Level of the upper critical band
        f1, f2 = upper_critical_band(ft)
        low_limit = np.argmin(np.abs(freqs - f1))
        high_limit = np.argmin(np.abs(freqs - f2))

        spec_sum = sum(10 ** (spec_db[low_limit:high_limit] / 10))
        if spec_sum != 0:
            Lu = 10 * np.log10(spec_sum)
        else:
            Lu = 0

        if ft <= 171.4:
            delta = 10 * np.log10(10 ** (0.1 * Lm)) - 10 * np.log10(
                ((100 / delta_f) * 10 ** (0.1 * Ll) + 10 ** (0.1 * Lu)) * 0.5
            )

        elif ft > 171.4:
            delta = 10 * np.log10(10 ** (0.1 * Lm)) - 10 * np.log10(
                (10 ** (0.1 * Ll) + 10 ** (0.1 * Lu)) * 0.5
            )

        if delta > 0:
            pr = np.append(pr, delta)
            tones_freqs = np.append(tones_freqs, ft)

            # Prominent discrete tone criteria
            if ft >= 89.1 and ft <= 1000:
                if delta >= 9 + 10 * np.log10(1000 / ft):
                    prominence.append(True)
                else:
                    prominence.append(False)
            elif ft > 1000:
                if delta >= 9:
                    prominence.append(True)
                else:
                    prominence.append(False)

        # suppression from the list of tones of all the candidates belonging to the
        # same critical band
        sup = np.where((peak_index >= low_limit_idx) & (peak_index <= high_limit_idx))[
            0
        ]
        peak_index = np.delete(peak_index, sup)
        nb_tones -= len(sup)

    if sum(np.power(10, (pr / 10))) != 0:
        t_pr = 10 * np.log10(sum(np.power(10, (pr / 10))))
    else:
        t_pr = 0

    return tones_freqs, pr, prominence, t_pr



# -*- coding: utf-8 -*-
"""
@date Created on Fri May 22 2020
@author martin_g for Eomys
"""

# Standard library imports
import math
import numpy as np


def loudness_zwicker_lowpass_intp(loudness, tau, sample_rate):
    """1st order low-pass with linear interpolation of signal for
    increased precision

    Parameters
    ----------
    loudness : numpy.ndarray
        Loudness vs. time
    tau : float
        Filter parameter
    sample_rate : int
        Louness signal sampling frequency

    Outputs
    -------
    filt_loudness : numpy.ndarray
        Filtered loudness
    """
    filt_loudness = np.zeros(np.shape(loudness))
    # Factor for virtual upsampling/inner iterations
    lp_iter = 24

    num_samples = np.shape(loudness)[0]
    a1 = math.exp(-1 / (sample_rate * lp_iter * tau))
    b0 = 1 - a1
    y1 = 0

    for i in range(num_samples):
        x0 = loudness[i]
        y1 = b0 * x0 + a1 * y1
        filt_loudness[i] = y1

        # Linear interpolation steps between current and next sample
        if i < num_samples - 1:
            xd = (loudness[i + 1] - x0) / lp_iter
            # Inner iterations/interpolation
            for ii in range(lp_iter):
                x0 += xd
                y1 = b0 * x0 + a1 * y1
    return filt_loudness



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 08:59:34 2020

@author: wantysal
"""

# Standard library imports
import numpy as np
import pyuff
from scipy.io import wavfile, loadmat
from scipy.signal import resample

# Local import
from mosqito.functions.oct3filter.comp_third_spectrum import comp_third_spec


def load(is_stationary, file, calib=1, mat_signal="", mat_fs=""):
    """Extract the signal and its time axis from .wav or .uff file,
    resample the signal to 48 kHz, and affects its sampling frequency
    and time signal values.

    Parameters
    ----------
    is_stationary : boolean
        TRUE if the signal is stationary, FALSE if it is time-varying
    file : string
        string path to the signal file
    calib : float
        calibration factor for the signal to be in [pa]
    mat_signal : string
        in case of a .mat file, name of the signal variable
    mat_fs : string
        in case of a .mat file, name of the sampling frequency variable

    Outputs
    -------
    signal : numpy.array
        time signal values
    fs : integer
        sampling frequency
    """

    # load the .wav file content
    if file[-3:] == "wav" or file[-3:] == "WAV":
        fs, signal = wavfile.read(file)

        # calibration factor for the signal to be in Pa
        if isinstance(signal[0], np.int16):
            signal = calib * signal / (2 ** 15 - 1)
        elif isinstance(signal[0], np.int32):
            signal = calib * signal / (2 ** 31 - 1)
        elif isinstance(signal[0], np.float):
            signal = calib * signal

    # load the .uff file content
    elif file[-3:].lower() == "uff" or file[-3:].lower() == "unv":
        uff_file = pyuff.UFF(file)
        data = uff_file.read_sets()
        data.keys()

        # extract the signal values
        signal = data["data"]

        # calculate the sampling frequency
        fs = int(1 / data["abscissa_inc"])

    # load the .mat file content
    elif file[-3:] == "mat":
        matfile = loadmat(file)

        # extract the signal values and sampling frequency
        signal = matfile[mat_signal][:, 0]
        fs = matfile[mat_fs]
        fs = fs[:, 0]

    else:
        raise ValueError("""ERROR: only .wav .mat or .uff files are supported""")

    # resample to 48kHz to allow calculation
    if fs != 48000:
        signal = resample(signal, int(48000 * len(signal) / fs))
        fs = 48000
        print("Signal resampled to 48 kHz to allow calculation.")

    return signal, fs


def load2oct3(is_stationary, file, calib=1):
    """Load .wav signal and output its third-octave band spectrum

    Parameters
    ----------
    is_stationary: boolean
        True if the signal is stationary, False if it is time-varying
    file : string
        full path to the signal file
    calib : float
        calibration factor for the signal to be in [pa]


    Outputs
    -------
    spec : numpy.ndarray
        Third octave band spectrum of signal sig [dB re.2e-5 Pa]
    fpref : numpy.ndarray
        Corresponding preferred third octave band center frequencies
    """

    # Load the signal from its file
    signal, fs = load(is_stationary, file, calib)

    # Compute third-octave spectrum
    output = comp_third_spec(is_stationary, signal, fs)

    return output


def load2wav(
    is_stationary, file, sampling_freq, calib=1, encodage=16, mat_signal="", mat_fs=""
):
    """Load .uff or .mat file and create the corresponding .wav audio file

    Parameters
    ----------
    is_stationary: boolean
        True if the signal is stationary, False if it is time-varying
    file : string
        full path to the signal file
    sampling_freq : integer
        sampling frequency of the created .wav file
    calib : float
        calibration factor for the signal to be in [pa]
    encodage : integer
        encodage of the signal, 16 for np.int16, 32 for np.int32
    mat_signal : string
        in case of a .mat file, name of the signal variable
    mat_fs : string
        in case of a .mat file, name of the sampling frequency variable
    Output
    ------
    None
    """

    # Load the .uff file content
    if file[-3:] == "uff" or file[-3:] == "UFF":
        uff_file = pyuff.UFF(file)
        data = uff_file.read_sets()
        data.keys()

        # extract the signal values
        signal = data["data"]

        # calculate the sampling frequency
        fs = int(1 / data["abscissa_inc"])

    # Load the .mat file content
    elif file[-3:] == "mat":
        matfile = loadmat(file)

        # extract the signal values and sampling frequency
        signal = matfile[mat_signal][:, 0]
        fs = matfile[mat_fs]
        fs = fs[:, 0]

    else:
        raise ValueError("""ERROR: only .mat or .uff file are supported""")

    # Resample
    if fs != sampling_freq:
        signal = resample(signal, sampling_freq * int(len(signal) / fs))

    # calibration factor for the signal to be in Pa
    if encodage == 16:
        signal = signal * (2 ** 15 - 1) / calib
        signal = signal.astype(np.int16)

    elif encodage == 32:
        signal = signal * (2 ** 31 - 1) / calib
        signal = signal.astype(np.int32)
    # create the .wav file
    newfile = file[:-3] + "wav"
    wavfile.write(newfile, sampling_freq, signal)



# -*- coding: utf-8 -*-
"""
@date Created on Tue Feb 25 2020
@author martin_g for Eomys
"""

# Third party imports
import numpy as np

# Local application imports
from mosqito.functions.loudness_zwicker.loudness_zwicker_shared import (
    calc_main_loudness,
)
from mosqito.functions.loudness_zwicker.loudness_zwicker_shared import calc_slopes


def loudness_zwicker_stationary(spec_third, third_axis=[], field_type="free"):
    """Zwicker-loudness calculation for stationary signals

    Calculates the acoustic loudness according to Zwicker method for
    stationary signals.
    Normatice reference:
        ISO 532:1975 (method B)
        DIN 45631:1991
        ISO 532-1:2017 (method 1)
    The code is based on BASIC program published in "Program for
    calculating loudness according to DIN 45631 (ISO 532B)", E.Zwicker
    and H.Fastl, J.A.S.J (E) 12, 1 (1991).
    Note that due to normative continuity, as defined in the
    preceeding standards, the method is in accordance with
    ISO 226:1987 equal loudness contours (instead of ISO 226:2003)

    Parameters
    ----------
    spec_third : numpy.ndarray
        A third octave band spectrum [dB ref. 2e-5 Pa]
    third_axis : numpy.ndarray
        Normalized center frequency of third octave bands [Hz]
    field_type : str
        Type of soundfield corresponding to spec_third ("free" by
        default or "diffuse")

    Outputs
    -------
    N : float
        Calculated loudness [sones]
    N_specific : numpy.ndarray
        Specific loudness [sones/bark]
    """
    #
    # Input parameters control and formating
    fr = [
        25,
        31.5,
        40,
        50,
        63,
        80,
        100,
        125,
        160,
        200,
        250,
        315,
        400,
        500,
        630,
        800,
        1000,
        1250,
        1600,
        2000,
        2500,
        3150,
        4000,
        5000,
        6300,
        8000,
        10000,
        12500,
    ]
    if field_type != "diffuse" and field_type != "free":
        raise ValueError("ERROR: field_type should be either 'diffuse' or 'free'")
    if len(spec_third) != 28:
        raise ValueError("ERROR: spectrum must contains 28 third octave bands values")
    if len(third_axis) == 0:
        third_axis = fr
    elif (len(third_axis) == 28 and np.all(third_axis != fr)) or len(third_axis) < 28:
        raise ValueError(
            """ERROR: third_axis does not contains 1/3 oct between 25 and 
            12.5 kHz. Check the input parameters"""
        )
    else:
        # TODO: manage spectrum and third_axis longer than 28 (extract data
        # between 25 and 12.5 kHz)
        pass
    #    if (min(spec_third) < -60 or max(spec_third) > 120):
    # TODO: replace value below -60 by -60 and raise a warning
    #        raise ValueError(
    #            """ERROR: Third octave levels must be within interval
    #            [-60, 120] dB ref. 2e-5 Pa (for model validity) """
    #        )
    #
    #
    # Calculate main loudness
    Nm = calc_main_loudness(spec_third, field_type)
    #
    # Calculation of specific loudness pattern and integration of overall
    # loudness by attaching slopes towards higher frequencies
    N, N_specific = calc_slopes(Nm)

    return N, N_specific



# -*- coding: utf-8 -*-
"""
@date Created on Fri Mar 13 2020
@author martin_g for Eomys
"""


# Standard library imports
import numpy as np
from scipy import signal

# Local application imports
from mosqito.functions.oct3filter.oct3dsgn import oct3dsgn


def oct3level(sig, fs, fc, sig_type="stationary", dec_factor=24):
    """Calculate rms level of a signal in the third octave band fc

    Calculate the rms level of the signal "sig" in the third-octave
    band centered on frequency "fc". If "fc" is such that fc < fs/200,
    the signal is downsampled for better third-octave filter design.

    Parameters
    ----------
    sig : numpy.ndarray
        time signal [any unit]
    fs : float
        Sampling frequency [Hz]
    fc : float
        Filter exact center frequency [Hz]
    sig_type : str
        Type of signal ('stationary' or 'time-varying')
    dec_factor : int
        Time signal to RMS vs. time decimation factor

    Outputs
    -------
    level : numpy.ndarray
        Rms level of sig in the third octave band centered on fc
    """

    """
    For meaningful design results, center frequency used should 
    preferably be higher than fs/200 (source ???). The signal is
    then downsampled if fc < fs/200. The procedure is inspired by 
    script GenerateFilters.m by Aaron Hastings, Herrick Labs, 
    Purdue University (version: 31 Oct 00)

    TODO: generalize to dec_factor that is not divisor of fs
            (using resample instead of decimate)
    """

    # Check for Nyquist-Shannon criteria
    if fc > 0.88 * (fs / 2):
        raise ValueError(
            """ERROR: Design not possible. Filter center frequency shall
            verify: fc <= 0.88 * (fs / 2)"""
        )
    # Check if dec_factor is a divisor of fs
    fs_divisor = [n for n in range(1, (fs + 1)) if fs % n == 0]
    if (sig_type != "stationary") and (not dec_factor in fs_divisor):
        raise ValueError(
            """ERROR: Design not possible. Time decimation factor shall
            be a divisor of fs"""
        )

    # Downsampling to a multiple of fs to respect "fc > fs/200"
    # (in case of time varying signal, the decimation factor shall be
    # a multiple of dec_factor)
    if sig_type == "stationary":
        dec_factors = fs_divisor
    else:
        dec_factors = [n for n in fs_divisor if dec_factor % n == 0]
    for i in dec_factors:
        fs_sub = fs / i
        if fs_sub / fc < 200:
            break
    dec_factor = int(dec_factor / i)
    if i == dec_factors[-1] and fs_sub / fc >= 200:
        raise ValueError(
            """ERROR: Design not possible. No time decimation factor that 
            satisfies fs_sub / (fc) > 200 have been found"""
        )
    # Generate the 1/3 oct. digital filter
    b, a = oct3dsgn(fc, fs_sub, n=3)
    # Downsample the signal
    if fs != fs_sub:
        sig = signal.decimate(sig, int(fs / fs_sub))
    # Filter the signal
    sig_filt = signal.lfilter(b, a, sig)
    if sig_type == "stationary":
        # Calculate overall rms level
        level = np.sqrt(sum(sig_filt ** 2) / len(sig_filt))
    else:
        # Calculate rms level versus time
        n_level = int(np.floor(sig_filt.shape[0] / dec_factor))
        sig_reshaped = sig_filt[: dec_factor * n_level].reshape((n_level, dec_factor))
        level = np.sqrt(np.mean(sig_reshaped ** 2, 1))
        if sig_filt.shape[0] % dec_factor != 0:
            level = np.append(
                level, np.sqrt(np.mean(sig_filt[dec_factor * n_level :] ** 2))
            )
    return level



# -*- coding: utf-8 -*-
"""
@date Created on Thu Mar 26 2020
@author martin_g for Eomys
"""

# Standard library imports
import numpy as np

# Local applications imports
from mosqito.functions.loudness_zwicker.loudness_zwicker_shared import (
    calc_main_loudness,
)
from mosqito.functions.loudness_zwicker.loudness_zwicker_nonlinear_decay import (
    calc_nl_loudness,
)
from mosqito.functions.loudness_zwicker.loudness_zwicker_shared import calc_slopes
from mosqito.functions.loudness_zwicker.loudness_zwicker_temporal_weighting import (
    loudness_zwicker_temporal_weighting,
)


def loudness_zwicker_time(third_octave_levels, field_type):
    """Calculate Zwicker-loudness for time-varying signals

    Calculate the acoustic loudness according to Zwicker method for
    time-varying signals.
    Normatice reference:
        DIN 45631/A1:2010
        ISO 532-1:2017 (method 2)
    The code is based on C program source code published alongside
    with ISO 532-1 standard.
    Note that for reasons of normative continuity, as defined in the
    preceeding standards, the method is in accordance with
    ISO 226:1987 equal loudness contours (instead of ISO 226:2003)

    Parameters
    ----------
    third_octave_levels : numpy.ndarray
        rms acoustic pressure [Pa] per third octave versus time
        (temporal resolution = 0.5ms)
    field_type : str
        Type of soundfield corresponding to signal ("free" by
        default or "diffuse")

    Outputs
    -------
    N : float
        Calculated loudness [sones]
    N_specific : numpy.ndarray
        Specific loudness [sones/bark]
    bark_axis : numpy.ndarray
        Corresponding bark axis
    """

    # Calculate core loudness
    num_sample_level = np.shape(third_octave_levels)[1]
    core_loudness = np.zeros((21, num_sample_level))
    for i in np.arange(num_sample_level - 1):
        core_loudness[:, i] = calc_main_loudness(third_octave_levels[:, i], field_type)
    #
    # Nonlinearity
    core_loudness = calc_nl_loudness(core_loudness)
    #
    # Calculation of specific loudness
    loudness = np.zeros(np.shape(core_loudness)[1])
    spec_loudness = np.zeros((240, np.shape(core_loudness)[1]))
    for i_time in np.arange(np.shape(core_loudness)[1]):
        loudness[i_time], spec_loudness[:, i_time] = calc_slopes(
            core_loudness[:, i_time]
        )
    #
    # temporal weigthing
    filt_loudness = loudness_zwicker_temporal_weighting(loudness)
    #
    # Decimation from temporal resolution 0.5 ms to 2ms and return
    dec_factor = 4
    N = filt_loudness[::dec_factor]
    N_spec = spec_loudness[:, ::dec_factor]
    return N, N_spec



# -*- coding: utf-8 -*-
"""
@date Created on Fri May 22 2020
@author martin_g for Eomys
"""

# Standard library imports
import math

# Third party import
import numpy as np


def calc_nl_loudness(core_loudness):
    """Simulate the nonlinear temporal decay of the hearing system

    Parameters
    ----------
    core_loudness : numpy.ndarray
        Core loudness

    Outputs
    -------
    nl_loudness :  numpy.ndarray
        Loudness with non linear temporal decay
    """
    # Initialization
    sample_rate = 2000
    nl_loudness = core_loudness
    # Factor for virtual upsampling/inner iterations
    nl_iter = 24
    # Time constants for non_linear temporal decay
    t_short = 0.005
    t_long = 0.015
    t_var = 0.075
    # Initializes constants B and states of capacitors C1 and C2
    delta_t = 1 / (sample_rate * nl_iter)
    P = (t_var + t_long) / (t_var * t_short)
    Q = 1 / (t_short * t_var)
    lambda_1 = -P / 2 + math.sqrt(P * P / 4 - Q)
    lambda_2 = -P / 2 - math.sqrt(P * P / 4 - Q)
    den = t_var * (lambda_1 - lambda_2)
    e1 = math.exp(lambda_1 * delta_t)
    e2 = math.exp(lambda_2 * delta_t)
    B = [
        (e1 - e2) / den,
        ((t_var * lambda_2 + 1) * e1 - (t_var * lambda_1 + 1) * e2) / den,
        ((t_var * lambda_1 + 1) * e1 - (t_var * lambda_2 + 1) * e2) / den,
        (t_var * lambda_1 + 1) * (t_var * lambda_2 + 1) * (e1 - e2) / den,
        math.exp(-delta_t / t_long),
        math.exp(-delta_t / t_var),
    ]
    nl_lp = {"B": B}

    for i_cl in np.arange(np.shape(core_loudness)[0]):
        # At beginning capacitors C1 and C2 are discharged
        nl_lp["uo_last"] = 0
        nl_lp["u2_last"] = 0

        for i_time in np.arange(np.shape(core_loudness)[1] - 1):
            # interpolation steps between current and next sample
            delta = (
                core_loudness[i_cl, i_time + 1] - core_loudness[i_cl, i_time]
            ) / nl_iter
            ui = core_loudness[i_cl, i_time]
            nl_lp = calc_nl_lp(ui, nl_lp)
            nl_loudness[i_cl, i_time] = nl_lp["uo_last"]
            ui += delta

            # inner iterations
            for i_in in np.arange(1, nl_iter):
                nl_lp = calc_nl_lp(ui, nl_lp)
                ui += delta
        nl_lp = calc_nl_lp(core_loudness[i_cl, i_time + 1], nl_lp)
        nl_loudness[i_cl, i_time + 1] = nl_lp["uo_last"]
    return core_loudness


def calc_nl_lp(ui, nl_lp):
    """Calculates Uo(t) from Ui(t) using UoLast and U2Last

    Parameters
    ----------
    ui : float
        TODO: description cf. standard
    nl_lp : dict
        Parameters for non_linear temporal decay
    Outputs
    -------
    nl_lp : dict
        Updated parameters for non_linear temporal decay
    """
    if ui < nl_lp["uo_last"]:  # case 1 (discharge)
        if nl_lp["uo_last"] > nl_lp["u2_last"]:  # case 1.1
            u2 = nl_lp["uo_last"] * nl_lp["B"][0] - nl_lp["u2_last"] * nl_lp["B"][1]
            uo = nl_lp["uo_last"] * nl_lp["B"][2] - nl_lp["u2_last"] * nl_lp["B"][3]
            if uo < ui:  # uo can't become
                uo = ui  # lower than ui
            if u2 > uo:  # u2 can't become
                u2 = uo  # higher than uo
        else:  # case 1.2
            uo = nl_lp["uo_last"] * nl_lp["B"][4]
            if uo < ui:  # uo can't become
                uo = ui  # lower than ui
            u2 = uo
    else:
        if abs(ui - nl_lp["uo_last"] < 1e-5):  # case 2 (charge)
            uo = ui
            if uo > nl_lp["u2_last"]:  # case 2.1
                u2 = (nl_lp["u2_last"] - ui) * nl_lp["B"][5] + ui
            else:  # case 2.2
                u2 = ui
        else:
            uo = ui
            u2 = (nl_lp["u2_last"] - ui) * nl_lp["B"][5] + ui
    # Preparation for next step
    nl_lp["uo_last"] = uo
    nl_lp["u2_last"] = u2

    return nl_lp



# -*- coding: utf-8 -*-
"""
Created on Thu Sep 10 12:22:23 2020

@author: wantysal
"""

# Standard library import
import numpy as np


def comp_sharpness_aures(N, N_specific, is_stationary):
    """Sharpness calculation

    The code is based on W. Aures' equation to determine sharpness
    from specific zwicker loudness.

    Parameters
    ----------
    N : float
        loudness value
    N_specific : np.ndarray
        specific critical bands loudness
    is_stationary : boolean
        indicates if the signal is stationary or time-varying

    Outputs
    -------
    S : sharpness
    """

    # Bark axis
    z = np.linspace(0.1, 24, int(24 / 0.1))

    if is_stationary == True:
        if N == 0:
            S = 0
        else:
            # weighting function depending on the loudness
            gA = np.zeros((z.size))
            gA = 0.078 * (np.exp(0.171 * z) / z) * (N / np.log(N * 0.05 + 1))
            S = 0.11 * sum(N_specific * gA * z * 0.1) / N
            print("Aures sharpness:", "%.2f" % S, "acum")
    else:
        S = np.zeros((N.size))
        gA = np.zeros((z.size, N.size))
        for t in range(N.size):
            if N[t] >= 0.1:
                # weighting function depending on the loudness
                gA[:, t] = (
                    0.078 * (np.exp(0.171 * z) / z) * (N[t] / np.log(N[t] * 0.05 + 1))
                )
                S[t] = 0.11 * sum(N_specific[:, t] * gA[:, t] * z * 0.1) / N[t]

    return S



# -*- coding: utf-8 -*-
"""
Created on Mon Feb  8 10:39:49 2021

@author: wantysal
"""

import pytest

# Local application imports
from mosqito.functions.shared.load import load
from mosqito.functions.tonality_tnr_pr.comp_tnr import comp_tnr


@pytest.mark.tnr  # to skip or run PR test
def test_tnr():
    """Test function for the prominence ratio calculation of an audio signal

    Validation function for the Audio_signal class "comp_tnr" method with signal array
    as input. The input signals are generated using audacity.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Test signal as input for prominence ratio calculation
    # signals generated using audacity : white noise + tones at 200 and 2000 Hz
    # the first one is stationary, the second is time-varying
    signal = []

    signal.append(
        {
            "is_stationary": True,
            "tones freq": [200, 2000],
            "data_file": r"tests\input\white_noise_442_1768_Hz_stationary.wav",
        }
    )

    signal.append(
        {
            "is_stationary": False,
            "data_file": r"tests\input\white_noise_442_1768_Hz_varying.wav",
        }
    )

    for i in range(len(signal)):
        # Load signal
        audio, fs = load(signal[i]["is_stationary"], signal[i]["data_file"])
        # Compute tone-to-noise ratio
        tnr = comp_tnr(signal[i]["is_stationary"], audio, fs, prominence=True)



# -*- coding: utf-8 -*-
"""
@date Created on Mon May 25 2020
@author martin_g for Eomys
"""

# Third party imports
import numpy as np
import matplotlib.pyplot as plt
import pytest
from pandas import ExcelFile, read_excel

# Local application imports
from mosqito.functions.loudness_zwicker.loudness_zwicker_time import (
    loudness_zwicker_time,
)
from mosqito.functions.shared.load import load2oct3


@pytest.mark.loudness_zwtv  # to skip or run only loudness zwicker time-varying tests
def test_loudness_zwicker_time():
    """Test function for the script loudness_zwicker_time

    Test function for the script loudness_zwicker_time with
    .wav file as input. The input file is provided by ISO 532-1 annex
    B4 and B5, the compliance is assessed according to section 6.1 of the
    standard. One .png compliance plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """
    # Test signal as input for time-varying loudness
    # (from ISO 532-1 annex B4)
    signal = {
        "data_file": "tests/input/Test signal 6 (tone 250 Hz 30 dB - 80 dB).wav",
        "xls": "tests/input/Results and tests for synthetic signals (time varying loudness).xlsx",
        "tab": "Test signal 6",
        "N_specif_bark": 2.5,
        "field": "free",
    }

    # Load signal and compute third octave band spectrum
    third_spec = load2oct3(False, signal["data_file"], calib=2 * 2 ** 0.5)

    # Compute Loudness
    (
        N,
        N_specific,
    ) = loudness_zwicker_time(third_spec["values"], signal["field"])

    # Check ISO 532-1 compliance
    assert check_compliance(N, N_specific, signal)


def check_compliance(N, N_specific, iso_ref):
    """Check the comppiance of loudness calc. to ISO 532-1

    Check the compliance of the input data N and N_specific
    to section 6.1 of ISO 532-1 by using the reference data
    described in dictionary iso_ref.

    Parameters
    ----------
    N : float
        Calculated loudness [sones]
    N_specific : numpy.ndarray
        Specific loudness [sones/bark]
    bark_axis : numpy.ndarray
        Corresponding bark axis
    iso_ref : dict
        {
            "data_file": <Path to reference input signal>,
            "N_file": <Path to reference calculated loudness versus time>
            "N_specif_file": <Path to reference calculated specific loudness>
            "N_specif_bark": <Bark value of the reference calculation>
        }
        Dictionary containing link to ref. data

    Outputs
    -------
    tst : bool
        Compliance to the reference data
    """
    tst = 1

    # Load ISO reference outputs
    xls_file = ExcelFile(iso_ref["xls"])
    N_iso = np.transpose(
        read_excel(
            xls_file,
            sheet_name=iso_ref["tab"],
            header=None,
            skiprows=10,
            usecols="B",
            squeeze=True,
        ).to_numpy()
    )
    N_iso = N_iso[~np.isnan(N_iso)]
    N_specif_iso = np.transpose(
        read_excel(
            xls_file,
            sheet_name=iso_ref["tab"],
            header=None,
            skiprows=10,
            usecols="L",
            squeeze=True,
        ).to_numpy()
    )
    N_specif_iso = N_specif_iso[~np.isnan(N_specif_iso)]
    #
    # Correct eventual length difference between MOSQITO and ISO results
    # (1 samples max)
    if np.abs(N_iso.size - N.size) <= 1:
        i_max = np.min([N_iso.size, N.size])
        N_iso = N_iso[:i_max]
        N_specif_iso = N_specif_iso[:i_max]
        N = N[:i_max]
        N_specific = N_specific[:, :i_max]
    elif np.abs(N_iso.size - N.size) > 1:
        tst = 0

    if tst:
        #
        # Define time vector
        time = np.linspace(0, 0.002 * (N.size - 1), N.size)
        #
        # Formating
        tolerances = [[0.9, 0.95, 1.05, 1.1], [-0.2, -0.1, 0.1, 0.2]]
        style = ["solid", "dashed", "dashed", "solid"]
        lab = ["10% tolerance", "5% tolerance", "", ""]
        clrs = ["red", "orange", "orange", "red"]
        #
        # +/- 2ms temporal tolerance
        if sum(abs(N[1:] - N_iso[:-1])) < sum(abs(N - N_iso)):
            N = N[1:]
            N_specific = N_specific[:, 1:]
            N_iso = N_iso[:-1]
            N_specif_iso = N_specif_iso[:-1]
            time = time[:-1]
        elif sum(abs(N[:-1] - N_iso[1:])) < sum(abs(N - N_iso)):
            N = N[:-1]
            N_specific = N_specific[:, :-1]
            N_iso = N_iso[1:]
            N_specif_iso = N_specif_iso[1:]
            time = time[1:]

        # critical band rate scale
        bark_axis = np.linspace(0.1, 24, int(24 / 0.1))

        # Generate compliance plot for loudness and specific loudness
        comp = np.zeros((4, N.size))
        if iso_ref["N_specif_bark"] != -1:
            #
            # Find index of the iso reference signal bark value
            bark = iso_ref["N_specif_bark"]
            i_bark = np.where(
                (np.abs(bark_axis - bark)) == np.amin(np.abs(bark_axis - bark))
            )[0][0]
            #
            # Data to plot
            Ni = [N, N_specific[i_bark, :]]
            Ni_ref = [N_iso, N_specif_iso]
            Ni_label = ["Loudness", "Specific loudness at " + str(bark) + " Bark"]
        else:
            #
            # Data to plot
            Ni = [N]
            Ni_ref = [N_iso]
            Ni_label = ["Loudness"]
        for N, N_ref, N_label in zip(Ni, Ni_ref, Ni_label):
            for i in np.arange(4):
                #
                # Define the tolerance curves and build compliance matrix
                if i in [0, 1]:
                    tol_curve = np.amin(
                        [N_ref * tolerances[0][i], N_ref + tolerances[1][i]], axis=0
                    )
                    comp[i, :] = N >= tol_curve
                else:
                    tol_curve = np.amax(
                        [N_ref * tolerances[0][i], N_ref + tolerances[1][i]], axis=0
                    )
                    comp[i, :] = N <= tol_curve
                tol_curve[tol_curve < 0] = 0
                #
                # Plot tolerance curves
                plt.plot(
                    time,
                    tol_curve,
                    color=clrs[i],
                    linestyle=style[i],
                    label=lab[i],
                    linewidth=1,
                )
            #
            # Check compliance
            comp_10 = np.array([comp[0, i] and comp[3, i] for i in np.arange(N.size)])
            comp_5 = np.array([comp[1, i] and comp[2, i] for i in np.arange(N.size)])
            ind_10 = np.nonzero(comp_10 == 0)[0]
            ind_5 = np.nonzero(comp_5 == 0)[0]
            if ind_5.size == 0:
                plt.text(
                    0.5,
                    0.5,
                    "Test passed (5% tolerance not exceeded)",
                    horizontalalignment="center",
                    verticalalignment="center",
                    transform=plt.gca().transAxes,
                    bbox=dict(facecolor="green", alpha=0.3),
                )
            elif ind_5.size / N.size <= 0.01:
                plt.text(
                    0.5,
                    0.5,
                    "Test passed (5% tolerance exceeded in maximum 1% of time)",
                    horizontalalignment="center",
                    verticalalignment="center",
                    transform=plt.gca().transAxes,
                    bbox=dict(facecolor="orange", alpha=0.3),
                    wrap=True,
                )
            else:
                tst = 0
                plt.text(
                    0.5,
                    0.5,
                    "Test not passed",
                    horizontalalignment="center",
                    verticalalignment="center",
                    transform=plt.gca().transAxes,
                    bbox=dict(facecolor="red", alpha=0.3),
                )
            #
            # Highlights non-compliant area
            for i in ind_10:
                plt.axvspan(
                    time[i] - 0.001, time[i] + 0.001, facecolor="red", alpha=0.3
                )
            for i in ind_5:
                if not i in ind_10:
                    plt.axvspan(
                        time[i] - 0.001, time[i] + 0.001, facecolor="orange", alpha=0.3
                    )
            #
            # Plot the calculated loudness
            plt.plot(time, N, label="MOSQITO")
            plt.title(
                N_label
                + " vs. time - "
                + iso_ref["data_file"].split("(")[1].split(")")[0]
                + " ("
                + iso_ref["data_file"].split("Test ")[1].split(" (")[0]
                + ")",
                fontsize=10,
            )
            plt.legend()
            plt.xlabel("Time [s]")
            if N_label == "Loudness":
                plt.ylabel("Loudness, [sone]")
            else:
                plt.ylabel("Specific loudness [sone/Bark]")
            file_name = "_".join(iso_ref["data_file"].split(" "))
            if tst:
                flag = ""
            else:
                flag = "FAILED_"
            plt.savefig(
                "tests/output/"
                + flag
                + "test_loudness_zwicker_time_"
                + file_name.split("/")[-1][:-4]
                + "_"
                + N_label.split(" ")[0]
                + ".png",
                format="png",
            )
            plt.clf()
    return tst



# -*- coding: utf-8 -*-
"""
Created on Thu Sep 10 16:59:53 2020

@author: wantysal
"""
# Standard library import
import numpy as np


def comp_sharpness_din(N, N_specific, is_stationary):
    """Sharpness calculation

    The code is based on DIN 45692:2009 to determine sharpness
    from specific zwicker loudness.

    Parameters
    ----------
    N : float
        loudness value
    N_specific : np.ndarray
        specific critical bands loudness
    is_stationary : boolean
        indicates if the signal is stationary or time-varying
    time : np.array
        time axis

    Outputs
    -------
    S : sharpness
    """

    # Bark axis
    z = np.linspace(0.1, 24, int(24 / 0.1))

    # weighting function
    gDIN = np.zeros((z.size))
    gDIN[z <= 15.8] = 1
    gDIN[z > 15.8] = 0.15 * np.exp(0.42 * (z[z > 15.8] - 15.8)) + 0.85

    if is_stationary:
        if N == 0:
            S = 0
        else:
            S = 0.11 * sum(N_specific * gDIN * z * 0.1) / N
            print("DIN sharpness:", "%.2f" % S, "acum")
    else:
        S = np.zeros((N.size))
        for t in range(N.size):
            if N[t] >= 0.1:
                S[t] = 0.11 * sum(N_specific[:, t] * gDIN * z * 0.1) / N[t]
    return S



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 09:10:45 2020

@author: wantysal
"""

# Standard import
import numpy as np

# Local imports
from mosqito.functions.oct3filter.calc_third_octave_levels import (
    calc_third_octave_levels,
)
from mosqito.functions.oct3filter.oct3spec import oct3spec


def comp_third_spec(is_stationary, signal, fs):
    """Third-octave band spectrum calculation, with the corresponding
    bands center frequencies

    Parameters
    ----------
    is_stationary: boolean
        TRUE if the signal is stationary, FALSE if it is time-varying
    signal : numpy.array
        time signal values
    fs : integer
        sampling frequency


    Outputs
    --------
    spec_third : numpy.ndarray
        Third octave band spectrum of signal sig [dB re.2e-5 Pa]
    spec_third_freq : numpy.ndarray
        Corresponding third octave bands center frequencies
    """

    if is_stationary == True:
        spec_third, third_axis = oct3spec(signal, fs)
        time_axis = []
    elif is_stationary == False:
        spec_third, third_axis, time_axis = calc_third_octave_levels(signal, fs)
    np.squeeze(spec_third)

    output = {
        "name": "Third-octave-spectrum",
        "values": spec_third,
        "freqs": third_axis,
        "time": time_axis,
    }

    return output



# -*- coding: utf-8 -*-
"""
@date Created on Fri Mar 13 2020
@author martin_g for Eomys
"""

# Standard library imports
import numpy as np

# Local application imports
from mosqito.functions.oct3filter.oct3level import oct3level


def oct3spec(sig, fs, fc_min=25, fc_max=12500, sig_type="stationary", dec_factor=24):
    """Calculate third-octave band spectrum

    Calculate the rms level of the signal "sig" sampled at freqency "fs"
    for each third octave band between "fc_min" and "fc_max".

    Parameters
    ----------
    sig : numpy.ndarray
        time signal [Pa]
    fs : float
        Sampling frequency [Hz]
    fc_min : float
        Filter center frequency of the lowest 1/3 oct. band [Hz]
    fc_max : float
        Filter center frequency of the highest 1/3 oct. band [Hz]
    out_format : str
        Format of the output data ("rms" by default or "time")
        corresponding respectively to a vector of overall rms values
        per third octave band or a matrix (dim [freq, time]) filtered
        time signals per third octave band
    sig_type : str
        Type of signal ('stationary' or 'time-varying'), influences the
        format of the output data corresponding respectively to a vector
        of overall rms values per third octave band or a matrix
        (dim [freq, time]) filtered time signals per third octave band
    fs_level : int
        RMS vs. time (pseudo-)sampling frequency.

    Outputs
    -------
    spec : numpy.ndarray
        Third octave band spectrum of signal sig [dB re.2e-5 Pa]
    fpref : numpy.ndarray
        Corresponding preferred third octave band center frequencies
    """

    # TODO: control that fc_min and fc_max are in the right range
    # TODO: smarter management of the frequencies by using the ANSI
    #       definitions (with base 10 and base 2 options)

    # Définition of the range of preferred filter center frequency
    fpref = np.array(
        [
            16,
            20,
            25,
            31.5,
            40,
            50,
            63,
            80,
            100,
            125,
            160,
            200,
            250,
            315,
            400,
            500,
            630,
            800,
            1000,
            1250,
            1600,
            2000,
            2500,
            3150,
            4000,
            5000,
            6300,
            8000,
            10000,
            12500,
            16000,
            20000,
        ]
    )

    fexact = np.array(
        [
            15.849,
            19.953,
            25.119,
            31.623,
            39.811,
            50.119,
            63.096,
            79.433,
            100,
            125.89,
            158.49,
            199.53,
            251.19,
            316.23,
            398.11,
            501.19,
            630.96,
            794.33,
            1000,
            1258.9,
            1584.9,
            1995.3,
            2511.9,
            3162.3,
            3981.1,
            5011.9,
            6309.6,
            7943,
            10000,
            12589,
            15849,
            19953,
        ]
    )

    fexact = fexact[fpref >= fc_min]
    fpref = fpref[fpref >= fc_min]
    fexact = fexact[fpref <= fc_max]
    fpref = fpref[fpref <= fc_max]

    # Calculation of the rms level of the signal in each band
    if sig_type == "stationary":
        spec = np.zeros((len(fexact), 1))
    else:
        n_level = int(np.ceil(sig.shape[0] / dec_factor))
        spec = np.zeros((len(fexact), n_level))
    i = 0
    for fc in fexact:
        spec[i, :] = oct3level(sig, fs, fc, sig_type, dec_factor)
        i += 1

    spec = 20 * np.log10((spec + 1e-12) / (2 * 10 ** -5))
    return spec, fpref



# -*- coding: utf-8 -*-

from scipy.signal import welch

from SciDataTool import Data1D, DataFreq


def compute_welch(self, N=None, noverlap=None):

    f, Pxx = welch(
        self.signal.values,
        fs=self.fs,
        nperseg=N,
        noverlap=noverlap,
        scaling="spectrum",
    )

    # Define axis objects
    frequency = Data1D(
        name="freqs",
        unit="Hz",
        values=f,
    )

    # Define Data object
    self.welch = DataFreq(
        name="Audio signal",
        symbol="x",
        axes=[frequency],
        values=Pxx ** 0.5,
        unit="Pa",
        normalizations={"ref": 2e-5},
    )



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 10:00:50 2020

@author: wantysal
"""

# Standard library import
import numpy as np

# Local imports
from mosqito.functions.loudness_zwicker.comp_loudness import comp_loudness
from mosqito.functions.sharpness.sharpness_aures import comp_sharpness_aures
from mosqito.functions.sharpness.sharpness_din import comp_sharpness_din
from mosqito.functions.sharpness.sharpness_bismarck import comp_sharpness_bismarck
from mosqito.functions.sharpness.sharpness_fastl import comp_sharpness_fastl


def comp_sharpness(is_stationary, signal, fs, method="din", skip=0):
    """Acoustic sharpness calculation according to different methods:
        Aures, Von Bismarck, DIN 45692, Fastl

    Parameters:
    ----------
    is_stationary: boolean
        True if the signal is stationary, false if it is time varying
    signal: numpy.array
        time history values
    fs: integer
        sampling frequency
    method: string
        'din' by default,'aures', 'bismarck','fastl'
    skip : float
        number of second to be cut at the beginning of the analysis

    Outputs
    ------
    S : float
    sharpness value

    """

    if (
        method != "din"
        and method != "aures"
        and method != "fastl"
        and method != "bismarck"
    ):
        raise ValueError("ERROR: method must be 'din', 'aures', 'bismarck', 'fastl'")

    loudness = comp_loudness(is_stationary, signal, fs)

    if method == "din":
        S = comp_sharpness_din(
            loudness["values"], loudness["specific values"], is_stationary
        )

    elif method == "aures":
        S = comp_sharpness_aures(
            loudness["values"], loudness["specific values"], is_stationary
        )

    elif method == "bismarck":
        S = comp_sharpness_bismarck(
            loudness["values"], loudness["specific values"], is_stationary
        )

    elif method == "fastl":
        S = comp_sharpness_fastl(
            loudness["values"], loudness["specific values"], is_stationary
        )

    if is_stationary == False:
        # Cut transient effect
        time = np.linspace(0, len(signal) / fs, len(S))
        cut_index = np.argmin(np.abs(time - skip))
        S = S[cut_index:]

    output = {"name": "sharpness", "method": method, "values": S, "skip": skip}

    return output



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 16 09:23:56 2020

@author: martin_g for eomys
"""

# Standard import
import numpy as np

# Local imports
from mosqito.functions.oct3filter.comp_third_spectrum import comp_third_spec
from mosqito.functions.loudness_zwicker.loudness_zwicker_stationary import (
    loudness_zwicker_stationary,
)
from mosqito.functions.loudness_zwicker.loudness_zwicker_time import (
    loudness_zwicker_time,
)


def comp_loudness(is_stationary, signal, fs, field_type="free"):
    """Acoustic loudness calculation according to Zwicker method for
    stationary and time-varying signals.

    Parameters
    ----------
    is_stationary: boolean
        TRUE if the signal is stationary, FALSE if it is time-varying
    signal : numpy.array
        time signal values
    fs : integer
        sampling frequency
    field-type: string
        'free' by default or 'diffuse'

    Outputs
    -------
    N: float/numpy.array
        loudness value
    N_specific: numpy.array
        specific loudness values
    bark_axis: numpy.array
        frequency axis correpsondong to N_specific values in bark
    """

    third_spec = comp_third_spec(is_stationary, signal, fs)

    if is_stationary == True:
        N, N_specific = loudness_zwicker_stationary(
            third_spec["values"], third_spec["freqs"], field_type
        )
    elif is_stationary == False:
        N, N_specific = loudness_zwicker_time(third_spec["values"], field_type)

    # critical band rate scale
    bark_axis = np.linspace(0.1, 24, int(24 / 0.1))

    output = {
        "name": "Loudness",
        "values": N,
        "specific values": N_specific,
        "freqs": bark_axis,
    }

    return output


def comp_loudness_from_3spec(
    is_stationary, third_spec, third_axis=[], field_type="free"
):
    """Acoustic loudness calculation according to Zwicker method for
    stationary and time-varying signals.

    Parameters
    ----------
    is_stationary: boolean
        TRUE if the signal is stationary, FALSE if it is time-varying
    third_spec: numpy.array
        third octave spectrum
    third_axis: numpy.array
        third-octav frequency axis
    field-type: string
        'free' by default or 'diffuse'

    Outputs
    -------
    N: float/numpy.array
        loudness value
    N_specific: numpy.array
        specific loudness values
    bark_axis: numpy.array
        frequency axis correpsondong to N_specific values in bark
    """

    if is_stationary == True:
        N, N_specific = loudness_zwicker_stationary(third_spec, third_axis, field_type)
    elif is_stationary == False:
        N, N_specific = loudness_zwicker_time(third_spec, field_type)

    if third_axis == []:
        # critical band rate scale
        third_axis = np.linspace(0.1, 24, int(24 / 0.1))

    output = {
        "name": "Loudness",
        "values": N,
        "specific values": N_specific,
        "freqs": third_axis,
    }

    return output



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 16 09:20:41 2020

@author: wantysal
"""

# Standard imports
import numpy as np
import pytest

# Local application imports
from mosqito.functions.shared.load import load
from mosqito.functions.sharpness.comp_sharpness import comp_sharpness


@pytest.mark.sharpness_din  # to skip or run sharpness test
def test_sharpness():
    """Test function for the sharpness calculation of an audio signal

    Test function for the function "comp_roughness" with 'din' method.
    The input signals come from DIN 45692_2009E. The compliance is assessed
    according to chapter 6 of the standard.
    One .png compliance plot is generated.

    Parameters
    ----------
    None

    Outputs
    -------
    None
    """

    # Input signal from DIN 45692_2009E
    signal = {"data_file": r"tests\input\1KHZ60DB.wav", "S": 1}

    # Load signal
    sig, fs = load(True, signal["data_file"], calib=1)

    # Compute sharpness
    sharpness = comp_sharpness(True, sig, fs, method="din")
    S = sharpness["values"]

    assert check_compliance(S, signal)


def check_compliance(S, signal):
    """Check the comppiance of loudness calc. to ISO 532-1

    The compliance is assessed according to chapter 6 of the
    standard DIN 45692_2009E.

    Parameters
    ----------
    S : float
        computed sharpness value
    signal : dict
        {"data file" : <path to the standard file >
         "S" : <sharpness reference value>
         }

    Outputs
    -------
    tst : bool
        Compliance to the reference data
    """

    # Load reference value
    ref = signal["S"]

    # Test for DIN 45692_2009E comformance (chapter 6)
    tst = (S >= np.amax([ref * 0.95, ref - 0.05], axis=0)).all() and (
        S <= np.amin([ref * 1.05, ref + 0.05], axis=0)
    ).all()

    return tst


# test de la fonction
if __name__ == "__main__":
    test_sharpness()



# -*- coding: utf-8 -*-
"""
Created on Wed Sep 16 15:47:08 2020

@author: pc
"""

import numpy as np


def A_weighting(spectrum, freq_axis):
    """A_weighting dB ponderation of a spectrum according to CEI 61672:2014

    Third-octave spectrum are directly calculated, other are calculated
    using linear interpolation.

    Parameters
    ----------
    spectrum: numpy.array
              input spectrum
    fs: integer
        sampling frequency

    """

    # Ponderation coefficients from the standard
    A_standard = np.array(
        [
            -70.4,
            -63.4,
            -56.7,
            -50.5,
            -44.7,
            -39.4,
            -34.6,
            -30.2,
            -26.2,
            -22.5,
            -19.1,
            -16.1,
            -13.4,
            -10.9,
            -8.6,
            -6.6,
            -4.8,
            -3.2,
            -1.9,
            -0.8,
            0,
            0.6,
            1,
            1.2,
            1.3,
            1.2,
            1,
            0.5,
            -0.1,
            -1.1,
            -2.5,
            -4.3,
            -6.6,
            -9.3,
        ]
    )

    freq_standard = np.array(
        [
            10,
            12.5,
            16,
            20,
            25,
            31.5,
            40,
            50,
            63,
            80,
            100,
            125,
            160,
            200,
            250,
            315,
            400,
            500,
            630,
            800,
            1000,
            1250,
            1600,
            2000,
            2500,
            3150,
            4000,
            5000,
            6300,
            8000,
            10000,
            12500,
            16000,
            20000,
        ]
    )

    A_pond = np.interp(freq_axis, freq_standard, A_standard)

    spectrum_dBA = np.zeros(spectrum.shape)
    for i in range(spectrum.shape[0]):
        spectrum_dBA[i] = spectrum[i] + A_pond[i]

    return spectrum_dBA



# -*- coding: utf-8 -*-
"""
Created on Thu Dec 17 09:19:35 2020

@author: wantysal
"""


def cut_signal(signal, fs, start, stop):
    """Cut the input signal before 'start' and after 'stop'

    Parameters
    ----------
    signal : numpy.array
        time signal values
    fs : integer
        sampling frequency
    start : float
        beginning of the new signal in [s]
    stop : float
        end of the new signal in [s]

    """
    return signal[int(start * fs) : int(stop * fs)]



# -*- coding: utf-8 -*-
"""
Created on Tue Feb 23 14:15:28 2021

@author: Salomé
"""
# Import SciDataTool objects
from SciDataTool import Data1D, DataFreq

# import Mosqito functions
from mosqito.functions.tonality_tnr_pr.comp_tnr import comp_tnr
from mosqito.functions.tonality_tnr_pr.comp_pr import comp_pr


def compute_tnr_pr(self, method, prominence=True):
    """Method to compute tonality metrics according to the given method

    Parameter
    ---------
    method : string
        'tnr' for the tone-to-noise ratio,
        'pr' for the prominence ratio,
        'all' for both
    prominence : boolean
        give only the prominent tones
    """

    if method == "tnr" or method == "all":
        T = comp_tnr(
            self.is_stationary,
            self.signal.values,
            self.fs,
            prominence=prominence,
        )

        freqs = Data1D(symbol="F", name="freqs", unit="Hz", values=T["freqs"])

        if self.is_stationary == True:

            self.tonality["tnr"] = DataFreq(
                symbol="TNR",
                axes=[freqs],
                values=T["values"],
                name="Tone-to-noise ratio",
                unit="dB",
            )

            self.tonality["ttnr"] = T["global value"]

        elif self.is_stationary == False:

            time = Data1D(symbol="T", name="time", unit="s", values=T["time"])

            self.tonality["tnr"] = DataFreq(
                symbol="TNR",
                axes=[freqs, time],
                values=T["values"],
                name="Tone-to-noise ratio",
                unit="dB",
            )

            self.tonality["ttnr"] = T["global value"]

    if method == "pr" or method == "all":
        T = comp_pr(
            self.is_stationary,
            self.signal.values,
            self.fs,
            prominence=prominence,
        )

        freqs = Data1D(symbol="F", name="freqs", unit="Hz", values=T["freqs"])

        if self.is_stationary == True:

            self.tonality["pr"] = DataFreq(
                symbol="PR",
                axes=[freqs],
                values=T["values"],
                name="Prominence ratio",
                unit="dB",
            )

            self.tonality["tpr"] = T["global value"]

        elif self.is_stationary == False:
            time = Data1D(symbol="T", name="time", unit="s", values=T["time"])

            self.tonality["pr"] = DataFreq(
                symbol="PR",
                axes=[freqs, time],
                values=T["values"],
                name="Prominence ratio",
                unit="dB",
            )

            self.tonality["tpr"] = T["global value"]



# -*- coding: utf-8 -*-
"""
Created on Wed Feb 17 13:03:02 2021

@author: Salomé
"""
import numpy as np

# Import SciDataTool objects
from SciDataTool import DataTime, Data1D

# Import MOSQITO functions
from mosqito.functions.sharpness.sharpness_aures import comp_sharpness_aures
from mosqito.functions.sharpness.sharpness_din import comp_sharpness_din
from mosqito.functions.sharpness.sharpness_bismarck import comp_sharpness_bismarck
from mosqito.functions.sharpness.sharpness_fastl import comp_sharpness_fastl


def compute_sharpness(self, method="din", skip=0.2):
    """Method to cumpute the sharpness according to the given method

    Parameter
    ---------
    method: string
        'din' by default, 'aures', 'bismarck', 'fastl', 'all'
    skip : float
        number of second to be cut at the beginning of the analysis

    """
    # check the input parameters
    if (
        method != "din"
        and method != "aures"
        and method != "fastl"
        and method != "bismarck"
        and method != "all"
    ):
        raise ValueError("ERROR: method must be 'din', 'aures', 'bismarck' or 'fastl")

    if skip < 0 or skip > (len(self.signal.values) / self.fs):
        raise ValueError(
            "ERROR: skip must be positive and inferior to the signal duration"
        )

    # compute the loudness by zwicker if not already done
    if self.loudness_zwicker == None:
        self.compute_loudness()

    if method == "din" or method == "all":

        if self.is_stationary == True:
            # Compute sharpness
            S = comp_sharpness_din(
                self.loudness_zwicker,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            self.sharpness["din"] = S

        elif self.is_stationary == False:
            # Compute sharpness
            S = comp_sharpness_din(
                self.loudness_zwicker.values,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )
            # Cut transient effect
            cut_index = np.argmin(
                np.abs(self.loudness_zwicker.get_axes()[0].values - skip)
            )
            S = S[cut_index:]

            # Define time axis
            time = Data1D(
                symbol="T",
                name="time",
                unit="s",
                values=self.loudness_zwicker.get_axes()[0].values[cut_index:],
            )

            self.sharpness["din"] = DataTime(
                symbol="S_{DIN}", axes=[time], values=S, name="Sharpness", unit="Acum"
            )

    elif method == "aures" or method == "all":

        if self.is_stationary == True:
            # Compute sharpness
            S = comp_sharpness_aures(
                self.loudness_zwicker,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            self.sharpness["aures"] = S

        elif self.is_stationary == False:

            # Compute sharpness
            S = comp_sharpness_aures(
                self.loudness_zwicker.values,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            # Cut transient effect
            cut_index = np.argmin(
                np.abs(self.loudness_zwicker.get_axes()[0].values - skip)
            )
            S = S[cut_index:]

            # Define time axis
            time = Data1D(
                symbol="T",
                name="time",
                unit="s",
                values=self.loudness_zwicker.get_axes()[0].values[cut_index:],
            )

            self.sharpness["aures"] = DataTime(
                symbol="S_{Aures}", axes=[time], values=S, name="Sharpness", unit="Acum"
            )

    elif method == "bismarck" or method == "all":

        if self.is_stationary == True:
            # Compute sharpness
            S = comp_sharpness_bismarck(
                self.loudness_zwicker,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            self.sharpness["bismarck"] = S

        elif self.is_stationary == False:
            # Compute sharpness
            S = comp_sharpness_bismarck(
                self.loudness_zwicker.values,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            # Cut transient effect
            cut_index = np.argmin(
                np.abs(self.loudness_zwicker.get_axes()[0].values - skip)
            )
            S = S[cut_index:]

            # Define time axis
            time = Data1D(
                symbol="T",
                name="time",
                unit="s",
                values=self.loudness_zwicker.get_axes()[0].values[cut_index:],
            )

            self.sharpness["bismarck"] = DataTime(
                symbol="S_{Bismarck}",
                axes=[time],
                values=S,
                name="Sharpness",
                unit="Acum",
            )

    elif method == "fastl" or method == "all":

        if self.is_stationary == True:
            # Compute sharpness
            S = comp_sharpness_fastl(
                self.loudness_zwicker,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            self.sharpness["fastl"] = S

        elif self.is_stationary == False:
            # Compute sharpness
            S = comp_sharpness_fastl(
                self.loudness_zwicker.values,
                self.loudness_zwicker_specific.values,
                self.is_stationary,
            )

            # Cut transient effect
            cut_index = np.argmin(
                np.abs(self.loudness_zwicker.get_axes()[0].values - skip)
            )
            S = S[cut_index:]

            # Define time axis
            time = Data1D(
                symbol="T",
                name="time",
                unit="s",
                values=self.loudness_zwicker.get_axes()[0].values[cut_index:],
            )

            self.sharpness["fastl"] = DataTime(
                symbol="S_{Fastl}", axes=[time], values=S, name="Sharpness", unit="Acum"
            )



# -*- coding: utf-8 -*-

from numpy import squeeze

from SciDataTool import Data1D, DataTime, DataFreq

from mosqito.functions.shared.A_weighting import A_weighting
from mosqito.functions.oct3filter.calc_third_octave_levels import (
    calc_third_octave_levels,
)
from mosqito.functions.oct3filter.oct3spec import oct3spec


def comp_3oct_spec(
    self,
    fc_min=25,
    fc_max=12800,
):
    """Method to compute third-octave spectrum according to ISO

    Parameter
    ---------
    fc_min : float
        Filter center frequency of the lowest 1/3 oct. band [Hz]
    fc_max : float
        Filter center frequency of the highest 1/3 oct. band [Hz]

    """

    # Compute third octave band spectrum
    if self.is_stationary:
        third_spec, freq_val = oct3spec(
            self.signal.values, self.fs, fc_min=fc_min, fc_max=fc_max
        )
    else:
        third_spec, freq_val, time_val = calc_third_octave_levels(
            self.signal.values, self.fs
        )

    # dB -> lin
    # Todo clean all code related to db to lin, use SciDataTool if possible
    third_spec = 2e-5 * 10 ** (third_spec / 20)

    # Define axis objects
    frequency = Data1D(
        name="freqs",
        unit="Hz",
        values=freq_val,
    )
    axes = [frequency]
    if not self.is_stationary:
        time = Data1D(
            name="time",
            unit="s",
            values=time_val,
        )
        axes.append(time)

    # Define Data object
    self.third_spec = DataFreq(
        name="Audio signal",
        symbol="x",
        axes=axes,
        values=third_spec,
        unit="Pa",
        normalizations={"ref": 2e-5},
    )
